# SMOP compiler runtime support library
# Copyright 2014 Victor Leikehman

# MIT license
import numpy as np
import unittest
from runtime import *

class Getitem(unittest.TestCase):
    def setUp(self):
        self.a = matlabarray(arange_(1,20).reshape(4,5,order="F"))
    # 2-dim
    def test01(self):
        self.assertEqual(self.a[1,1],1)

    def test02(self):
        self.assertEqual(self.a[4,5],20)

    def test03(self):
        self.assertTrue(isequal_(self.a[:,:],self.a))

    # 1-dim
    def test04(self):
        a = matlabarray(arange_(1,20).reshape(4,5,order="F"))
        aa = a[:]
        bb = matlabarray(arange_(1,20).reshape(-1,1,order="F"))
        self.assertTrue(isequal_(aa,bb))

    def test05(self):
        #import pdb; pdb.set_trace()
        z = [[11,22,33,44],
             [11,22,33,44],
             [11,22,33,44],
             [11,22,33,44]]
        a = matlabarray([11,22,33,44], dtype=int)
        self.assertTrue(isequal_(a[  [1,1,1,1] , 1:4], matlabarray(z)))
        self.assertTrue(isequal_(a[  [1,1,1,1],   : ], matlabarray(z)))
        #self.assertTrue(isequal_(a[ [[1,1,1,1]], 1:4], matlabarray([z])))
            
    def test06(self):
        a=copy_(0)
        a[6]=666
        self.assertTrue(isequal_(a, [[0.,0.,0.,0.,0.,666.]]))

class Expand(unittest.TestCase):
    """
    Expand on index error  
    """
    def setUp(self):
        self.a = matlabarray(zeros_(1,4))

    def test01(self):
        self.a[1,5]=1
        self.assertTrue(isequal_(self.a,
                                 matlabarray([[0,0,0,0,1]])))

    def test02(self):
        #with self.assertRaises(IndexError):
        a = matlabarray(zeros_(1,4))
        a[5]=1 # single index
        self.assertTrue(isequal_(a,
                                 matlabarray([[0,0,0,0,1]])))

    #@unittest.skip("")
    def test03(self):
        a=zeros_(1,4) 
        a[1:10:4]=1
        "[[ 1.  0.  0.  0.  1.  0.  0.  0.  1.  0.]]"

    #@unittest.skip("")
    def test04(self):
        a=zeros_(1,4) 
        with self.assertRaises(IndexError):
            a[5,5]=1
            b = matlabarray(
                [[0,0,0,0,0],
                 [0,0,0,0,0],
                 [0,0,0,0,0],
                 [0,0,0,0,0],
                 [0,0,0,0,1]])
            self.assertTrue(isequal_(a,b))
 
class Strread(unittest.TestCase):
    def test01(self):
        a = strread_("0.11 0.22 0.33")
        self.assertTrue(isequal_(a,[[0.11,0.22,0.33]]))

    def test02(self):
        a,b,c = strread_("0.11 0.22 0.33",nargout=3)
        self.assertEqual(a,0.11)
        self.assertEqual(b,0.22)
        self.assertEqual(c,0.33)

class Core(unittest.TestCase):
    def setUp(self):
        self.a = arange_(1,10).reshape(2,5,order="F")

    def test01(self):
        b = abs_(-self.a)
        self.assertTrue(isequal_(self.a,b))

    def test02(self):
        b = ceil_(self.a)
        self.assertTrue(isequal_(self.a,b))

    def test03(self):
        b = false_(2,3)
        self.assertTrue(isequal_(size_(b), [[2,3]]))

    def test_zeros(self):
        self.assertEqual(zeros_(), 0.0)
        self.assertTrue(isequal_(zeros_(2), zeros_(2,2)))
        self.assertTrue(isequal_(zeros_(2,2), zeros_(2,2)))

        
#class Copy(unittest.TestCase):
#    def setUp(self):
#        self.a = zeros_(1,4)
#
#    def test01(self):
#        b=self.a.copy()
#        print self.a
#        print b
#        self.assertTrue(isequal_(self.a,b))
#        b[1,1]=123
#        self.assertTrue(not isequal_(self.a,b))
#        #c=b.copy()
#        #c[1,:]=1
#        #self.assertTrue(not isequal_(b,c))

if __name__ == "__main__":
    unittest.main()



"""
Given node.function instance, apply it
to argument list, such as ["i","d"]
and return the result.
"""

import node,options
from node import extend,exceptions

callstack = set()

@extend(node.function)
@exceptions
def apply(self,args,symtab):
    name = self.head.ident.name 
    if name in callstack:
        return
    print "%%%", self.head.ident.name
    callstack.add(name)

    params = [(u.name,u.lineno) for u in self.head.args]
    if len(args) < len(self.head.args): # head.args are formal params
        args += [''] * (len(self.head.args)-len(args))
    symtab.update(zip(params,args))
    self.body._typeof(symtab)
    #return [symtab[u.name,u.lineno] for u in self.ret] 
#    for u in node.postorder(self):
#        if u.__class__ is node.ident:
#            try:
#                print u.name,u.lineno,symtab[u.name,u.lineno]
#            except:
#                print u.name,u.lineno,'?'
    return '' #[u._typeof(symtab) for u in self.head.ret]

@extend(node.node)
@exceptions
def _typeof(self,symtab):
    for u in self:
        u._typeof(symtab)

@extend(node.for_stmt)
@exceptions
def _typeof(self,symtab):
    symtab[self.ident.name,self.ident.lineno] = 'i'
    self.stmt_list._typeof(symtab)

@extend(node.func_decl)
@exceptions
def _typeof(self,symtab):
    pass # ignore ident,args,ret

@extend(node.let)
@exceptions
def _typeof(self,symtab):
    if self.ret.__class__ is node.ident:
        symtab[self.ret.name,self.ret.lineno] = self.args._typeof(symtab)

@extend(node.param)
@extend(node.cellfun)  # FIXME
@exceptions
def _typeof(self,symtab):
    return ''

@extend(node.ident)
@exceptions
def _typeof(self,symtab):
    """
    symtab maps pairs (name,lineno) to string
    encoding of type.
    """
    if self.defs is None:
        try:
            return symtab[self.name,self.lineno]
        except:
            print '+++ Missing type for',self.name
            return '' 
    ts = set([u._typeof(symtab) for u in self.defs if u.defs is None])
    if '' in ts:
        ts.remove('')
    if 'i' in ts and 'd' in ts:
        ts.remove('i')
    if len(ts) == 0:
        #print '+++','No definition of variable',self.name,self.lineno
        return ''
    if len(ts) == 1:
        self._t = ts.pop()
        return self._t
    if len(ts) > 1:
        print '+++','Conflicting defs for', self.name,self.lineno,ts
        return ''
        
    return ''

@extend(node.matrix)
@exceptions
def _typeof(self,symtab):
    ts = set([u._typeof(symtab) for u in self.args if u._typeof(symtab)])
    if len(ts) >= 1:
        return ts.pop()
    return 'd'

#    ts = set([u._typeof(symtab) for u in self])
#    if len(ts) == 1:
#        return ts.pop()
#    return ''

@extend(node.arrayref) 
@extend(node.cellarrayref) 
@exceptions
def _typeof(self,symtab):
    return self.func_expr._typeof(symtab)

@extend(node.expr)
@exceptions
def _typeof(self,symtab):
    nargs = len(self.args)
    if nargs == 0:
        assert 0
    if nargs == 1:
        if self.op in ("!","~"):
            return 'l'
        if self.op in ("-","parens"):
            return self.args[0]._typeof(symtab)
        assert 0, self.op
    if len(self.args) == 2:
        if self.op == ":":
            return 'i'
        if self.op in ("<",">","<=",">=","==","~=","!=","&&","||"):
            return 'l'
        if self.op in ("*","+","-","/","^",".*","./"):
            if (self.args[0]._typeof(symtab) == 'i' and 
                self.args[1]._typeof(symtab) == 'i'):
                return 'i'
            else:
                return 'd'

@extend(node.add)
@extend(node.sub)
@exceptions
def _typeof(self,symtab):
    if (self.args[0]._typeof(symtab) == 'i' and 
        self.args[1]._typeof(symtab) == 'i'):
        return 'i'
    else:
        return 'd'

@extend(node.ones)
@extend(node.zeros)
@exceptions
def _typeof(self,symtab):
    if self.args[-1]._typeof(symtab) == 'c':
        raise Exception("Only 'double' is implemented")
    return 'i'

@extend(node.RESHAPE)
@extend(node.MAXVAL)
@extend(node.MINVAL)
@extend(node.CEILING)
@extend(node.abs)
@extend(node.ceil)
@extend(node.sign)
@extend(node.min)
@extend(node.max)
@extend(node.transpose)
@exceptions
def _typeof(self,symtab):
    return self.args[0]._typeof(symtab)

# sum is different from similar funcs.
# re-check this
@extend(node.sum)
@extend(node.cumsum)
@extend(node.cumprod)
@exceptions
def _typeof(self,symtab):
    if self.args[0]._typeof(symtab) == 'l':
        return 'i'
    return self.args[0]._typeof(symtab)

@extend(node.number)
@exceptions
def _typeof(self,symtab):
    return 'i' if isinstance(self.value,int) else 'd'

@extend(node.diff)
@extend(node.RAND)
@extend(node.rand)
@extend(node.inf)
@extend(node.dot)
@extend(node.nnz)
@extend(node.mod)
@exceptions
def _typeof(self,symtab):
    return 'd'

@extend(node.rem)
@extend(node.SIZE)
@extend(node.size)
@extend(node.find)
#@extend(node.findone)
@exceptions
def _typeof(self,symtab):
    return 'ii'

@extend(node.sort)
@exceptions
def _typeof(self,symtab):
    return self.args[0]._typeof(symtab)
    #if len(self.ret) == 2:
    #    return self.args[0]._typeof(symtab) + 'i'

@extend(node.numel)
@extend(node.floor)
@extend(node.length)
@extend(node.range_)
@exceptions
def _typeof(self,symtab):
    return 'i'

@extend(node.false)
@extend(node.true)
@extend(node.isequal)
@extend(node.isnan)
@extend(node.isinf)
@extend(node.isempty)
@extend(node.all)
@extend(node.any)
@exceptions
def _typeof(self,symtab):
    return 'l'

@extend(node.funcall) # func_expr,args
@exceptions
def _typeof(self,symtab):
    func_obj = symtab[self.func_expr.name]
    #if not self.ret:
    return func_obj.apply(self.args,symtab)
    #return ['' for i in self.ret]

@extend(node.ravel)
@exceptions
def _typeof(self,symtab):
    return self.args[0]._typeof(symtab)

@extend(node.continue_stmt)
@extend(node.break_stmt)
@exceptions
def _typeof(self,symtab):
    pass



# smop -- Simple Matlab to Python compiler
# Copyright 2011-2014 Victor Leikehman

"""
Calling conventions:

call site:  nargout=N is passed if and only if N > 1
func decl:  nargout=1 must be declared if function may return
            more than one return value.  Otherwise optional.
return value:  return (x,y,z)[:nargout] or return x
"""

import logging
logger = logging.getLogger(__name__)
import node,options
from node import extend

indent = " "*4

optable = {
    "!" : "not",
    "~" : "not",
    "~=": "!=",
    "|" : "or",
    "&" : "and",
    "||": "or",
    "&&": "and",
    "^" : "**",
    ".^": "**",
    "./": "/",
    ".*": "*",
    }

def backend(t,*args):
    return t._backend(*args)

@extend(node.matrix)
def _backend(self,level=0):
    # TODO empty array has shape of 0 0 in matlab
    # size([])
    # 0 0
    if not self.args:
        return "[]"
    elif any(a.__class__ is node.string for a in self.args):
        return " + ".join(a._backend() for a in self.args)
    else:
        #import pdb; pdb.set_trace()
        return "[%s]" % self.args[0]._backend()

@extend(node.cellarrayref)
def _backend(self,level=0):
    return "%s[%s]" % (self.func_expr._backend(),
                       self.args._backend())
@extend(node.cellarray)
def _backend(self,level=0):
    return "[%s]" % self.args._backend()

#@extend(node.concat_list)
#def _backend(self,level=0):
#    return ";".join([t._backend() for t in self])

@extend(node.ravel)
def _backend(self,level=0):
    return "%s.ravel()" % self.args[0]._backend()

@extend(node.transpose)
def _backend(self,level=0):
    return "%s.T" % self.args[0]._backend()

@extend(node.expr_stmt)
def _backend(self,level=0):
    return self.expr._backend()

@extend(node.return_stmt)
def _backend(self,level=0):
    if not self.ret:
        return "return" 
    else:
        return "return %s" % self.ret._backend()

@extend(node.continue_stmt)
def _backend(self,level=0):
    return "continue"

@extend(node.global_stmt)
def _backend(self,level=0):
    return "global %s" % self.global_list._backend()

@extend(node.global_list)
def _backend(self,level=0):
    return ",".join([t._backend() for t in self])

@extend(node.break_stmt)
def _backend(self,level=0):
    return "break"

@extend(node.string)
def _backend(self,level=0):
    return repr(self.value)

@extend(node.number)
def _backend(self,level=0):
    #if type(self.value) == int:
    #    return "%s.0" % self.value
    return str(self.value)

@extend(node.logical)
def _backend(self,level=0):
    if self.value == 0:
        return "false"
    else:
        return "true"
    
# @extend(node.range)
# def _backend(self,level=0):
#     i = node.ident.new("I")
#     return "[ (%s, %s=%s,%s) ]" % (i,i,self.args[0],self.args[1])

@extend(node.add)
def _backend(self,level=0):
    if (self.args[0].__class__ is node.number and
        self.args[1].__class__ is node.number):
        return node.number(self.args[0].value +
                           self.args[1].value)._backend()
    else:
        return "(%s+%s)" % (self.args[0]._backend(),
                            self.args[1]._backend())

@extend(node.sub)
def _backend(self,level=0):
    return "(%s-%s)" %  (self.args[0]._backend(),
                         self.args[1]._backend())

@extend(node.expr)
def _backend(self,level=0):
    if self.op == '@': # FIXME
        return self.args[0]._backend()

    if self.op == "\\":
        return "numpy.linalg.solve(%s,%s)" % (self.args[0]._backend(),
                                              self.args[1]._backend())
    if self.op == "::":
        if not self.args:
            return ":"
        elif len(self.args) == 2:
            return "%s:%s" % (self.args[0]._backend(),
                              self.args[1]._backend())
        elif len(self.args) == 3:
            return "%s:%s:%s" % (self.args[0]._backend(),
                                 self.args[2]._backend(),
                                 self.args[1]._backend())
    if self.op == ":":
        return "arange_(%s)" % self.args._backend()
    
    if self.op == "end":
        if self.args:
            return "%s.shape[%s]" % (self.args[0]._backend(),
                                     self.args[1]._backend())
        else:
            return "end"
    if self.op == ".":
        return "%s%s" % (self.args[0]._backend(),
                         self.args[1]._backend())
#     if self.op == "matrix":
#         return "[%s]" % ",".join([t._backend() for t in self.args])
    if self.op == "parens":
        return "(%s)" % self.args[0]._backend()
#    if self.op == "[]":
#        return "[%s]" % self.args._backend()
    if not self.args:
        return self.op
    if len(self.args) == 1:
        return "%s %s" % (optable.get(self.op,self.op),
                         self.args[0]._backend())
    if len(self.args) == 2:
        return "%s %s %s" % (self.args[0]._backend(),
                           optable.get(self.op,self.op),
                           self.args[1]._backend())
    #import pdb;pdb.set_trace()
    ret = "%s=" % self.ret._backend() if self.ret else ""
    return ret+"%s(%s)" % (self.op,
                           ",".join([t._backend() for t in self.args]))

@extend(node.arrayref)
def _backend(self,level=0):
#    if (len(self.args) == 1 and not
#        (self.args[0].__class__== node.expr and self.args[0].op=="::")):
#        fmt = "%s(%s)"
#    else:
    if options.subscripts == "round":
        fmt = "%s(%s)"
    elif options.subscripts == "square":
        fmt = "%s[%s]"
    else:
        assert False, options.subscripts
    return fmt % (self.func_expr._backend(),
                       self.args._backend())

@extend(node.funcall)
def _backend(self,level=0):
    #import pdb; pdb.set_trace()
    if not self.nargout or self.nargout == 1:
        return "%s(%s)" % (self.func_expr._backend(),
                           self.args._backend())
    elif not self.args:
        return "%s(nargout=%s)" % (self.func_expr._backend(),
                                   self.nargout)
    else:
        return "%s(%s,nargout=%s)" % (self.func_expr._backend(),
                                      self.args._backend(),
                                      self.nargout)

@extend(node.let)
def _backend(self,level=0):
#    if options.line_numbering:
#        s = "# %d\n" % self.lineno + level*indent
#    else:
#        s = ''
    #if self.args.__class__ is node.funcall:
    #    self.args.nargout = self.nargout
    if (self.ret.__class__ is node.ident and
        self.args.__class__ is node.ident):
        s = "%s=copy_(%s)" % (self.ret._backend(),
                              self.args._backend())
    else:
        s = "%s=%s" % (self.ret._backend(), 
                       self.args._backend())
    return s

@extend(node.expr_list)
def _backend(self,level=0):
    return ",".join([t._backend() for t in self])

@extend(node.concat_list)
def _backend(self,level=0):
    #import pdb; pdb.set_trace()
    return ",".join(["[%s]"%t._backend() for t in self])

# @extend(node.call_stmt)
# def _backend(self,level=0):
#     return "CALL %s(%s,%s)" % (self.func_expr._backend(),
#                                self.args._backend(),
# self.ret._backend())

fortran_type = {
    '@': '***',
    'd': 'DOUBLE PRECISION',
    'i': 'INTEGER',
    'l': 'LOGICAL',
    'c': 'CHARACTER',
}

# def decl__backend(i):
#     assert isinstance(i,ident)
#     try:
#         if i._rank() == 0:
#             return "%s :: %s\n" % (fortran_type[i._type()],
#                                    i)
#         return ("%s,DIMENSION(%s),ALLOCATABLE :: %s\n" % 
#                 (fortran_type[i._type()],
#                  ",".join([":" for j in range(i._rank())]), i))
#     except:
#         return "??? :: %s\n" % i
@extend(node.function)
def _backend(self,level=0):
    s = self.head._backend(level)
    t = self.body._backend(level+1)
    return "%s%s" %  (s,t)
        

# Sometimes variable names collide with _python reserved
# words and constants.  We handle this in the _backend rather than in
# the lexer, to keep the target language separate from
# the lexer code.
reserved = set(
    """
    abs all and any apply as assert basestring bin bool break buffer bytearray
    callable chr class classmethod cmp coerce compile complex continue copyright
    credits def del delattr dict dir divmod elif Ellipsis else enumerate eval
    except exec execfile exit False file filter finally float for format from
    frozenset getattr global globals hasattr hash help hex id if import __import__
    in input int intern is isinstance issubclass iter lambda len license list
    locals long map max memoryview min next None not not NotImplemented object oct
    open or ord pass pow print property quit raise range raw_input reduce reload
    repr return reversed round set setattr slice sorted staticmethod str sum super
    True try tuple type unichr unicode vars while with xrange yield zip
    """.split())

@extend(node.ident)
def _backend(self,level=0):
    return self.name if self.name not in reserved else "_"+self.name

@extend(node.stmt_list)
def _backend(self,level=0):
    sep = "\n"+indent*level
    if len(self):
        return sep+sep.join([t._backend(level) for t in self])
    else:
        return sep+"pass"

@extend(node.func_decl)
def _backend(self,level=0):
    if not self.use_nargin:
        if self.args:
            s = "def %s(%s,nargout=1):" %  (self.ident._backend(),
                                            self.args._backend())
        else:
            s = "def %s(nargout=1):" %  self.ident._backend()
    else:
        if not self.args:
            s = "def %s(*args,**kwargs):\n" % self.ident._backend()
        else:        
            s = "def %s(%s,*args,**kwargs):\n" % (self.ident._backend(),
                                                  self.args._backend())
        s += '''    varargin = cellarray(args)
    nargin = len(args)+%d''' % len(self.args)

    return s

"""
@extend(node.allocate_stmt)
def _backend(self,level=0):
    s = "ALLOCATE (%s(%s))" % (self.ident._backend(),
                               self.args._backend())
    return s
"""

@extend(node.lambda_expr)
def _backend(self,level=0):
    return 'lambda %s: %s' % (self.args._backend(),
                              self.ret._backend())
@extend(node.for_stmt)
def _backend(self,level=0):
    fmt = "for %s in %s.reshape(-1):%s"
    return fmt % (self.ident._backend(),
                  self.expr._backend(),
                  self.stmt_list._backend(level+1))

@extend(node.if_stmt)
def _backend(self,level=0):
    s = "if %s:%s" % (self.cond_expr._backend(),
                      self.then_stmt._backend(level+1))
    if self.else_stmt:
        # Eech. This should have been handled in the parser.
        if self.else_stmt.__class__ == node.if_stmt:
            self.else_stmt = node.stmt_list([self.else_stmt])
        s += "\n"+indent*level
        s += "else:%s" % self.else_stmt._backend(level+1)
    return s

@extend(node.while_stmt)
def _backend(self,level=0):
    fmt = "while %s:\n%s\n"
    return fmt % (self.cond_expr._backend(),
                  self.stmt_list._backend(level+1))

@extend(node.try_catch)
def _backend(self,level=0):
    fmt = "try:%s\n%sfinally:%s"
    return fmt % (self.try_stmt._backend(level+1),
                  indent*level,
                  self.finally_stmt._backend(level+1))

@extend(node.builtins)
def _backend(self,level=0):
    #if not self.ret:
        return "%s(%s)" % (self.__class__.__name__,
                           self.args._backend())
    # else:
    #     return ("%s=%s(%s)" % (self.ret._backend(),
    #                            self.__class__.__name__,
    #                            self.args._backend()))

### @extend(node.strcmp)
### def _backend(self,level=0):
###     return "%s==%s" % (self.args[0]._backend(),
###                        self.args[1]._backend())
### @extend(node.strcmpi)
### def _backend(self,level=0):
###     return "%s.lower()==%s.lower()" % (self.args[0]._backend(),
###                                        self.args[1]._backend())
                       
### @extend(node.isequal)
### def _backend(self,level=0):
###     return "np.array_equal(%s)" % self.args._backend()

### @extend(node.isempty)
### @exceptions
### def _backend(self,level=0):
###     return "(0 in %s.shape)" % self.args[0]._backend()

### @extend(node.numel)
### @exceptions
### def _backend(self,level=0):
###     return "%s.size" % self.args[0]._backend()

### @extend(node.size)
### @exceptions
### def _backend(self,level=0):
###     if len(self.args) == 1:
###         # if self.ret:
###         #     return "%s = %s.shape" % (self.ret,
###         #                               self.args[0]._backend())
###         # else:
###         return "%s.shape" % self.args[0]._backend()
###             
###     if self.args[1].__class__ is node.number:
###         return "%s.shape[%s]" % (self.args[0]._backend(),
###                                  self.args[1].value-1)
###     return "%s.shape[%s-1]" % (self.args[0]._backend(),
###                                self.args[1]._backend())

### @extend(node.cumsum)
### @exceptions
### def _backend(self,level=0):
###     if len(self.args) == 1:
###         return "np.cumsum(%s,0)" % self.args[0]._backend()
###     if self.args[1].__class__ is node.number:
###         return "np.cumsum(%s,%s)" % (self.args[0]._backend(),
###                                      self.args[1].value-1)
###     return "np.cumsum(%s,%s-1)" % (self.args[0]._backend(),
###                                    self.args[1]._backend())
    
@extend(node.dot)
def _backend(self,level=0):
    return "%s.dot(%s)" % (self.args[0]._backend(),
                           self.args[1]._backend())
### @extend(node.length)
### @exceptions
### def _backend(self,level=0):
###     return "max(%s.shape)" % self.args[0]._backend()

### @extend(node.fopen)
### @exceptions
### def _backend(self,level=0):
###     return "open(%s)" % self.args._backend()
### 
### @extend(node.fclose)
### @exceptions
### def _backend(self,level=0):
###     return "%s.close()" % self.args._backend()
### 
### @extend(node.min)
### @extend(node.max)
### @extend(node.sum)
### @extend(node.zeros)
### @exceptions
### def _backend(self,level=0):
###     cls_name = self.__class__.__name__
###     return ("np." + cls_name  + "(%s)") % self.args._backend()
### 
### @extend(node.exist)
### @exceptions
### def _backend(self,level=0):
###     if self.args[1].__class__ is node.string and self.args[1].value == "file":
###         return "os.path.exists(%s)" % self.args[0]._backend()
###     if self.args[1].__class__ is node.string and self.args[1].value == "var":
###         return "%s in globals()" % self.args[0]._backend()
###     raise NotImplementedError("Not implemented: exist")

### @extend(node.find)
### @exceptions
### def _backend(self,level=0):
###     return "np.flatnonzero(%s)" % self.args[0]._backend()
    # if self.ret and len(self.ret) == 2:
    #     return "%s,%s = np.nonzero(%s)" % (self.ret[0]._backend(),
    #                                        self.ret[1]._backend(),
    #                                        self.args[0]._backend())
    # else:
    #     return "np.flatnonzero(%s)" % self.args[0]._backend()

### @extend(node.load)
### @exceptions
### def _backend(self,level=0):
###     return "loadmat(%s,matlab_compatible=True)" % self.args._backend()
### 
### @extend(node.save)
### @exceptions
### def _backend(self,level=0):
###     return "savemat(%s)" % self.args._backend()



import unittest
import parse

class TestParse(unittest.TestCase):
    def test_p03(self):
        """Expected failure"""
        s = """[1 ; 1; 1 ; ];"""
        t = parse.parse(s)
        self.assert_(t)

    def test_p04(self):
        """Dot has higher precedence than other operations"""
        s = "a+b.c.d;"
        t = parse.parse(s)
        u = [parse.expr_stmt(id=1, expr=[('+', parse.ident(name='a',
        lineno=1, lexpos=0),
        parse.field(expr=parse.field(expr=parse.ident(name='b',
        lineno=1, lexpos=2), ident=parse.ident(name='.c', lineno=1,
        lexpos=3)), ident=parse.ident(name='.d', lineno=1, lexpos=5)))])]
        self.assertEqual(t,u)

#    def test_p05(self):
#        """Iterate over LHS nodes (TBD)"""
#        s = "[foo(A.x(B.y)).bar(C.z).bzz,hello.world] =1;"
#        t = parse.parse_buf(s)
#        u = ["foo",".bar",".bzz","hello",".world"]
#        self.assertEqual([v[1] for v in dataflow.lhs(t[1][1])],u)
#
    def test_p06(self):
        """Cell arrays"""
        s = """
        {1 ...
	'foo' ...
        'bar' ...
        'bzz'};
        """
        t = parse.parse(s)
        self.assert_(t)

if __name__ == "__main__":
    unittest.main()



import numpy as np
import unittest
from runtime import *
from sparsearray import sparsearray

class Slice(unittest.TestCase):
     def setUp(self):
         self.a = sparsearray(arange_(1,4).reshape(2,2,order="F"))

     def test01(self):
         #self.assertTrue(isequal_(self.a[1:4],[1,2,3,4]))
         self.assertEqual(self.a[1,1], 1)
         self.assertEqual(self.a[2,1], 2)
         self.assertEqual(self.a[1,2], 3)
         self.assertEqual(self.a[2,2], 4)
#
#    def test02(self):
#        self.a[1,1] = 11
#        self.a[2,1] = 22
#        self.a[1,2] = 33
#        self.a[2,2] = 44
#        self.assertTrue(isequal_(self.a[1:4],[11,22,33,44]))
#
#    def test03(self):
#        self.a[1] = 11
#        self.a[2] = 22
#        self.a[3] = 33
#        self.a[4] = 44
#        self.assertTrue(isequal_(self.a[1:4],[11,22,33,44]))
#
#    def test04(self):
#        #with self.assertRaises(ValueError):
#        self.a[1:4]=[11,22,33,44]
#        self.assertTrue(isequal_(self.a[1:4],[11,22,33,44]))
#
#    def test05(self):
#        self.a[:,:]=[[11,33],[22,44]]
#        self.assertTrue(isequal_(self.a[1:4],[11,22,33,44]))
#
#    def test06(self):
#        self.a[:]=[11,22,33,44]
#        self.assertTrue(isequal_(self.a[1:4],[11,22,33,44]))
#
#    def test07(self):
#        self.a[::3]=[11,44]
#        self.assertTrue(isequal_(self.a[1:4],[11,2,3,44]))
#
#    def test08(self):
#        self.a[1:4:3]=[11,44]
#        self.assertTrue(isequal_(self.a[1:4],[11,2,3,44]))
#
#    def test_007(self):
#        """
#        One-dimensional LHS indexing without expansion, 
#        using list index.
#        """
#
#        a[[1,4]]=1
#        self.assertEqual(str(a), "[[ 1.  0.]\n [ 0.  1.]]")
#
#    @unittest.skip("")
#    def test_008(self):
#        a=zeros_(2,2)
#        a[[4,3,2,1]]=[1,2,3,4]
#        self.assertEqual(str(a), "[[ 1.  2.]\n [ 3.  4.]]")
#
#
#
#    @unittest.skip("")
#    def test_010(self):
#        a=zeros_(2,2)
#        a[2,:]=[1,2]
#        self.assertEqual(str(a), "[[ 0.  0.]\n [ 1.  2.]]")
#
#    @unittest.skip("")
#    def test_011(self):
#        a=zeros_(2,2)
#        a[2,1:2]=[1,2]
#        self.assertEqual(str(a), "[[ 0.  0.]\n [ 1.  2.]]")
#
#    @unittest.skip("")
#    def test_012(self):
#        a=zeros_(2,2)
#        a[2,[1,2]]=[1,2]
#        self.assertEqual(str(a), "[[ 0.  0.]\n [ 1.  2.]]")
#
#    @unittest.skip("")
#    def test_013(self):
#        a=zeros_(2,2)
#        a[:,:]=[[1,2],[3,4]]
#        self.assertEqual(str(a), "[[ 1.  2.]\n [ 3.  4.]]")
#
#    @unittest.skip("")
#    def test_014(self):
#        a=zeros_(2,2)  
#        with self.assertRaises(ValueError):
#            a[:,:]=[1,2,3,4]

class Sparsearray(unittest.TestCase):
    def setUp(self):
        self.a = np.arange(1,5).reshape(2,2,order="F")
        self.b = sparsearray(self.a)

    def test01(self):
        self.assertEqual(self.b[1,1],1)
        self.assertEqual(self.b[2,1],2)
        self.assertEqual(self.b[1,2],3)
        self.assertEqual(self.b[2,2],4)

class Index(unittest.TestCase):
    def setUp(self):
        self.a = sparsearray(arange_(1,4).reshape(2,2,order="F"))

    def test01(self):
        """
        One-dimensional LHS indexing of multi-dimensional array,
        without expansion.  Using scalar index.
        """
        self.a[4]=9
        self.assertTrue(isequal_(self.a, sparsearray([[1, 3],[2, 9]])))

    def test02(self):
        """
        Multi-dimensional LHS indexing, without expansion.
        """
        self.a[1,1]=11
        self.a[2,2]=22
        self.a[1,2]=12
        self.a[2,1]=21
        self.assertTrue(isequal_(self.a, sparsearray([[11,12],[21,22]])))

class Indices(unittest.TestCase):
    def setUp(self):
        self.a = sparsearray(np.arange(1,5).reshape(2,2,order="F"))
        self.c = sparsearray(arange_(1,20).reshape(4,5,order="F"))

    def test01(self):
        with self.assertRaises(ValueError):
            self.a[100]=100
        self.a[3,3]=33
        self.assertTrue(isequal_(self.a.todense(), np.asarray([[1,3,0],[2,4,0],[0,0,33]])))

    def test02(self):
        self.assertTrue(isequal_(self.a[:], [1,2,3,4]))
        self.assertTrue(isequal_(self.a[1:4], [1,2,3,4]))

    @unittest.skip("")
    def test03(self):
        self.assertTrue(isequal_(self.a[:,:], [[1,3],[2,4]]))
        self.assertTrue(isequal_(self.a[1:2,1:2], [1,2,3,4]))

    def test04(self):
        a = sparsearray([1,2,3])
        a[5]=100
        a[3,3]=33
        self.assertEqual(a.shape, (3,5))
        self.assertTrue(isequal_(a.todense(), [[1,2,3,0,100],[0,0,0,0,0],[0,0,33,0,0]]))


class Slice(unittest.TestCase):
     def setUp(self):
         self.a = sparsearray(arange_(1,4).reshape(2,2,order="F"))

     def test01(self):
         #self.assertTrue(isequal_(self.a[1:4],[1,2,3,4]))
         self.assertEqual(self.a[1,1], 1)
         self.assertEqual(self.a[2,1], 2)
         self.assertEqual(self.a[1,2], 3)
         self.assertEqual(self.a[2,2], 4)

class Sparsearray(unittest.TestCase):
    def setUp(self):
        self.a = np.arange(1,5).reshape(2,2,order="F")
        self.b = sparsearray(self.a)

    def test01(self):
        self.assertEqual(self.b[1,1],1)
        self.assertEqual(self.b[2,1],2)
        self.assertEqual(self.b[1,2],3)
        self.assertEqual(self.b[2,2],4)

class Index(unittest.TestCase):
    def setUp(self):
        self.a = sparsearray(arange_(1,4).reshape(2,2,order="F"))

    def test01(self):
        """
        One-dimensional LHS indexing of multi-dimensional array,
        without expansion.  Using scalar index.
        """
        self.a[4]=9
        self.assertTrue(isequal_(self.a, sparsearray([[1, 3],[2, 9]])))

    def test02(self):
        """
        Multi-dimensional LHS indexing, without expansion.
        """
        self.a[1,1]=11
        self.a[2,2]=22
        self.a[1,2]=12
        self.a[2,1]=21
        self.assertTrue(isequal_(self.a, sparsearray([[11,12],[21,22]])))

class Indices(unittest.TestCase):
    def setUp(self):
        self.a = sparsearray(np.arange(1,5).reshape(2,2,order="F"))
        self.c = sparsearray(arange_(1,20).reshape(4,5,order="F"))

    def test01(self):
        with self.assertRaises(ValueError):
            self.a[100]=100
        self.a[3,3]=33
        self.assertTrue(isequal_(self.a.todense(), np.asarray([[1,3,0],[2,4,0],[0,0,33]])))

    def test02(self):
        self.assertTrue(isequal_(self.a[:], [1,2,3,4]))
        self.assertTrue(isequal_(self.a[1:4], [1,2,3,4]))

    @unittest.skip("")
    def test03(self):
        self.assertTrue(isequal_(self.a[:,:], [[1,3],[2,4]]))
        self.assertTrue(isequal_(self.a[1:2,1:2], [1,2,3,4]))

    def test04(self):
        a = sparsearray([1,2,3])
        a[5]=100
        a[3,3]=33
        self.assertEqual(a.shape, (3,5))
        self.assertTrue(isequal_(a.todense(), [[1,2,3,0,100],[0,0,0,0,0],[0,0,33,0,0]]))

if __name__ == "__main__":
    unittest.main()



import numpy
import a
from runtime import *

ai = matlabarray(zeros_(10,10,dtype=int))
af = copy_(ai)

ai[1,1]=2
ai[2,2]=3
ai[3,3]=4
ai[4,4]=5
ai[5,5]=1

af[9,9]=1
af[8,8]=2
af[7,7]=3
af[6,6]=4
af[10,10]=5

mv = a.solver_(ai,af,0)
print mv



__all__ = ['recordtype']

import sys
from textwrap import dedent
from keyword import iskeyword


def recordtype(typename, field_names, verbose=False, **default_kwds):
    '''Returns a new class with named fields.

    @keyword field_defaults: A mapping from (a subset of) field names to default
        values.
    @keyword default: If provided, the default value for all fields without an
        explicit default in `field_defaults`.

    >>> Point = recordtype('Point', 'x y', default=0)
    >>> Point.__doc__           # docstring for the new class
    'Point(x, y)'
    >>> Point()                 # instantiate with defaults
    Point(x=0, y=0)
    >>> p = Point(11, y=22)     # instantiate with positional args or keywords
    >>> p[0] + p.y              # accessible by name and index
    33
    >>> p.x = 100; p[1] =200    # modifiable by name and index
    >>> p
    Point(x=100, y=200)
    >>> x, y = p               # unpack
    >>> x, y
    (100, 200)
    >>> d = p.todict()         # convert to a dictionary
    >>> d['x']
    100
    >>> Point(**d) == p        # convert from a dictionary
    True
    '''
    # Parse and validate the field names.  Validation serves two purposes,
    # generating informative error messages and preventing template injection attacks.
    if isinstance(field_names, basestring):
        # names separated by whitespace and/or commas
        field_names = field_names.replace(',', ' ').split()
    field_names = tuple(map(str, field_names))
    if not field_names:
        raise ValueError('Records must have at least one field')
    for name in (typename,) + field_names:
        if not min(c.isalnum() or c=='_' for c in name):
            raise ValueError('Type names and field names can only contain '
                             'alphanumeric characters and underscores: %r' % name)
        if iskeyword(name):
            raise ValueError('Type names and field names cannot be a keyword: %r'
                             % name)
        if name[0].isdigit():
            raise ValueError('Type names and field names cannot start with a '
                             'number: %r' % name)
    seen_names = set()
    for name in field_names:
        if name.startswith('_'):
            raise ValueError('Field names cannot start with an underscore: %r'
                             % name)
        if name in seen_names:
            raise ValueError('Encountered duplicate field name: %r' % name)
        seen_names.add(name)
    # determine the func_defaults of __init__
    field_defaults = default_kwds.pop('field_defaults', {})
    if 'default' in default_kwds:
        default = default_kwds.pop('default')
        init_defaults = tuple(field_defaults.get(f,default) for f in field_names)
    elif not field_defaults:
        init_defaults = None
    else:
        default_fields = field_names[-len(field_defaults):]
        if set(default_fields) != set(field_defaults):
            raise ValueError('Missing default parameter values')
        init_defaults = tuple(field_defaults[f] for f in default_fields)
    if default_kwds:
        raise ValueError('Invalid keyword arguments: %s' % default_kwds)
    # Create and fill-in the class template
    numfields = len(field_names)
    argtxt = ', '.join(field_names)
    reprtxt = ', '.join('%s=%%r' % f for f in field_names)
    dicttxt = ', '.join('%r: self.%s' % (f,f) for f in field_names)
    tupletxt = repr(tuple('self.%s' % f for f in field_names)).replace("'",'')
    inittxt = '; '.join('self.%s=%s' % (f,f) for f in field_names)
    itertxt = '; '.join('yield self.%s' % f for f in field_names)
    eqtxt   = ' and '.join('self.%s==other.%s' % (f,f) for f in field_names)
    template = dedent('''
        class %(typename)s(object):
            '%(typename)s(%(argtxt)s)'

            __slots__  = %(field_names)r

            def __init__(self, %(argtxt)s):
                %(inittxt)s

            def __len__(self):
                return %(numfields)d

            def __iter__(self):
                %(itertxt)s

            def __getitem__(self, index):
                return getattr(self, self.__slots__[index])

            def __setitem__(self, index, value):
                return setattr(self, self.__slots__[index], value)

            def todict(self):
                'Return a new dict which maps field names to their values'
                return {%(dicttxt)s}

            def __repr__(self):
                return '%(typename)s(%(reprtxt)s)' %% %(tupletxt)s

            def __eq__(self, other):
                return isinstance(other, self.__class__) and %(eqtxt)s

            def __ne__(self, other):
                return not self==other

            def __getstate__(self):
                return %(tupletxt)s

            def __setstate__(self, state):
                %(tupletxt)s = state
    ''') % locals()
    # Execute the template string in a temporary namespace
    namespace = {}
    try:
        exec template in namespace
        if verbose: print template
    except SyntaxError, e:
        raise SyntaxError(e.message + ':\n' + template)
    cls = namespace[typename]
    cls.__init__.im_func.func_defaults = init_defaults
    # For pickling to work, the __module__ variable needs to be set to the frame
    # where the named tuple is created.  Bypass this step in enviroments where
    # sys._getframe is not defined (Jython for example).
    if hasattr(sys, '_getframe') and sys.platform != 'cli':
        cls.__module__ = sys._getframe(1).f_globals['__name__']
    return cls


if __name__ == '__main__':
    import doctest
    TestResults = recordtype('TestResults', 'failed, attempted')
    print TestResults(*doctest.testmod())



# SMOP compiler -- Simple Matlab/Octave to Python compiler
# Copyright 2011-2013 Victor Leikehman

from collections import namedtuple
from recipes import recordtype
import copy,sys,inspect
import options

# def preorder(u):
#     if isinstance(u,traversable):
#         yield u
#         for n in u:
#             for t in preorder(n):
#                 yield t

def decode(self):
    r = ""
    s = self.name
    while s:
        if len(s) >= 2 and s[0] == "_":
            r += s[1].upper()
            s = s[2:]
        else:
            r += s[0].lower()
            s = s[1:]
    return r

def encode(s):
    return "".join(c+"_" if c.isupper() or c=="_" else c.upper() for c in s)

def postorder(u):
    if isinstance(u,node):
        for v in u:
            for t in postorder(v):
                yield t
        yield u # returns only traversible objects

def extend(cls):
    return lambda f: (setattr(cls,f.__name__,f) or f)

def exceptions(f):
    def wrapper(self,*args,**kwargs):
        try:
            return f(self,*args,**kwargs)
        except:
            print "%s.%s()" % (self.__class__.__name__, f.__name__)
            raise
    wrapper.__name__ = f.__name__
    return wrapper

class node(object):
    def become(self,other):
        class Wrapper(self.__class__):
            def __copy__(self):
                other = object.__getattribute__(self,"other")
                return copy.copy(other)
            def __getattribute__(self,name):
                other = object.__getattribute__(self,"other")
                return getattr(other,name)
            def __setattr__(self,name,value):
                other = object.__getattribute__(self,"other")
                return setattr(other,name,value)
            def __iter__(self):
                other = object.__getattribute__(self,"other")
                return iter(other)
            #def __hash__(self):
            #    other = object.__getattribute__(self,"other")
            #    return id(other)
            def __repr__(self):
                other = object.__getattribute__(self,"other")
                return repr(other)
            def __len__(self):
                other = object.__getattribute__(self,"other")
                return len(other)
        assert self != other
        self.other = other
        self.__class__ = Wrapper

    def _type(self):
        raise AttributeError("_type")


######### LISTS

class concat_list(node,list):
    pass

class global_list(node,list):
    """space-separated list of variables used in GLOBAL statement"""
    pass

class expr_list(node,list):
    def __str__(self):
        return ",".join([str(t) for t in self])
    def __repr__(self):
        return "expr_list(%s)" % list.__repr__(self)

class stmt_list(node,list):
    def __str__(self):
        return "\n".join([str(t) for t in self])
    def __repr__(self):
        return "stmt_list(%s)" % list.__repr__(self)

#####################
#
#  ATOMS

class atom(node): pass

class string(atom,recordtype("string", "value lineno lexpos", default=None)):
    def __str__(self):
        return "'%s'" % self.value

class logical(atom,recordtype("logical", "value lineno lexpos", default=None)):
    pass

class number(atom,recordtype("number","value lineno lexpos",default=None)):
    def __str__(self):
        return str(self.value)

class ident(atom,recordtype("ident","name lineno column lexpos defs props",
        default=None)):
    def __str__(self):
        return self.name

class param(ident):
    pass

###########################
#
#  STATEMENTS
#

class stmt(node): pass

# class call_stmt(stmt,recordtype("call_stmt","func_expr args ret")):
#     """Sometimes called multiple assignment, call statements represent
#     something like [x,y]=foo(a,b,c); Note the square brackets around
#     the lhs.
#     SEE ALSO: funcall,let
#     """
#     def __str__(self):
#         return "%s=%s(%s)" % (str(self.ret),
#                               str(self.func_expr),
#                               str(self.args))

class let(stmt,recordtype("let",
                          "ret args lineno lexpos nargout",
                          default=None)):
    """Assignment statement, except [x,y]=foo(x,y,z),
    which is handled by call_stmt."""
    def __str__(self):
        return "%s=%s" % (str(self.ret), str(self.args))
    
class func_decl(stmt,recordtype("func_decl","ident ret args decl_list use_nargin",default=None)):
    pass

class lambda_expr(func_decl):
    pass

class function(stmt,recordtype("function","head body")):
    pass

class for_stmt(stmt,recordtype("for_stmt","ident expr stmt_list")):
    pass

class DO_STMT(stmt,recordtype("DO_STMT","ident start stop stmt_list")):
    pass

# We generate where_stmt to implement A(B==C) = D
class where_stmt(stmt,recordtype("where_stmt","cond_expr stmt_list")):
    pass

class if_stmt(stmt,recordtype("if_stmt","cond_expr then_stmt else_stmt")):
    pass

class global_stmt(stmt,recordtype("global_stmt","global_list")):
    def __str__(self):
        return "global %s" % str(self.global_list)

class return_stmt(stmt,namedtuple("return_stmt","ret")):
    def __str__(self):
        return "return"

class end_stmt(stmt,namedtuple("end_stmt","dummy")):
    def __str__(self):
        return "end"

class continue_stmt(stmt,namedtuple("continue_stmt","dummy")):
    def __str__(self):
        return "continue"

class break_stmt(stmt,namedtuple("break_stmt","dummy")):
    def __str__(self):
        return "break"

class expr_stmt(stmt,node,recordtype("expr_stmt","expr")):
    def __str__(self):
        return str(self.expr)

class while_stmt(stmt,node,recordtype("while_stmt","cond_expr stmt_list")):
    pass

class try_catch(stmt,recordtype("try_catch","try_stmt catch_stmt finally_stmt")):
    pass

class allocate_stmt(stmt,recordtype("allocate_stmt",
                                    "ident args")):
    pass

#######################################333
#
# FUNCALL

class funcall(node,recordtype("funcall","func_expr args nargout",default=None)):
    """Funcall instances represent 
    (a) Array references, both lhs and rhs
    (b) Function call expressions
    """
    def __str__(self):
        return "%s(%s)" % (str(self.func_expr),
                           str(self.args))

class builtins(funcall):
    """
    Builtin functions are represented as subclasses
    of class builtins.  Application of a function to
    specific arguments is represented as its instance.
    For example, builtin function foo is represented
    as class foo(builtins), and foo(x) is represented
    as foo(x).
    """

    def __init__(self,*args,**kwargs):
        """
        If a built-in function _foo takes three arguments
        a, b, and c, we can just say _foo(a,b,c) and let
        the constructor take care of proper structuring of
        the node (like expr_list around the arguments, etc.
        """
        funcall.__init__(self,
             func_expr=None,
             args=expr_list(args),
             **kwargs)
        #import pdb; pdb.set_trace()
        #self.func_expr.defs = set(self.func_expr)

    def __repr__(self):
        return "np.%s%s" % (self.__class__,repr(self.args))

    def __str__(self):
        return "np.%s(%s)" % (self.__class__.__name__,
                              str(self.args))

class arrayref(funcall):
    def __repr__(self):
        return "%s%s[%s]" % (self.__class__, 
                             self.func_expr,
                             self.args)


########################## EXPR

class expr(node,recordtype("expr","op args")):
    def __str__(self):
        if self.op == ".":
            return "%s%s" % (str(self.args[0]),self.args[1])
        if self.op == "parens":
            return "(%s)" % str(self.args[0])
        if not self.args:
            return str(self.op)
        if len(self.args) == 1:
            return "%s%s" % (self.op,self.args[0])
        if len(self.args) == 2:
            return "%s%s%s" % (self.args[0]._backend(),
                               self.op,
                               self.args[1]._backend())
        ret = "%s=" % str(self.ret) if self.ret else ""
        return ret+"%s(%s)" % (self.op,
                               ",".join([str(t) for t in self.args]))

# names in caps correspond to fortran funcs
builtins_list = [
    "ABS",
    "ALL",
    "ANY",
    "CEILING",
    "FIND",
    "ISNAN",
    "MAXVAL",
    "MINVAL",
    "MODULO",
    "RAND",
    "RESHAPE",
    "SHAPE",
    "SIGN",
    "SIZE",
    "SUM",

    #"abs",
    "add", # synthetic opcode
    #"all",
    #"any",
    "cellfun",
    #"ceil",
    "clazz",
    #"cumprod",
    #"cumsum",
    #"diff",
    "dot",      # Exists in numpy. Implements matlab .*
    #"exist",
    "false",
    #"fclose",
    #"find",
    #"findone",  # same as find, but returns ONE result
    #"floor",
    #"fopen",
    "getfield",
    "inf", "inf0",
    #"isempty",
    #"isequal",
    "isinf",
    "isnan",
    #"length",
    #"load",
    #"lower",
    #"max",
    #"min",
    #"mod",
    #"nnz",
    #"numel",
    #"ones",
    #"rand",
    #"range_",   # synthetic opcode
    "ravel",   # synthetic opcode
    #"rem",
    #"save",
    "setfield",
    #"sign",
    #"size",
    #"sort",
    #"strcmp",
    #"strcmpi",
    "sub", # synthetic opcode for subtract
    #"sum",
    "transpose",
    #"true",
    #"zeros",
]

for name in builtins_list:
    globals()[name] = type(name, (builtins,), {})

#class cellarrayref(node,recordtype("cellarrayref","ident args")):
class cellarrayref(funcall):
    pass

class cellarray(expr):
    pass

class matrix(builtins):
    """
    Anything enclosed in square brackets counts as matrix
    >>> print matrix([1,2,3])
    [1,2,3]
    >>> print matrix()
    []
    """
#    def __init__(self,args=expr_list()):
#        expr.__init__(self,op="[]",args=args)
#    def __str__(self):
#        return "[%s]" % ",".join([str(t) for t in self.args])

@extend(node)
def is_const(self):
    return False
@extend(number)
@extend(string)
def is_const(self):
    return True
@extend(expr_list)
def is_const(self):
    return all(t.is_const() for t in self)
@extend(matrix)
def is_const(self):
    return not self.args or self.args[0].is_const()
# vim: ts=8:sw=4:et



# SMOP compiler runtime support library
# Copyright 2014 Victor Leikehman

# MIT license

"""
Main differences between Matlab matrices and numpy arrays
=========================================================

#. Array indices start with one, not with zero.  Accordingly, the last
   element of an N-element array is N, not N-1 as in C.

#. Matrix elements are ordered column-first, aka "Fortran" order.

#. Arrays auto-expand on out-of-bound lhs indexing.

#. In matlab, arrays can be updated before they are created::

      clear a
      a(17) = 42

   is legal in matlab, but not in numpy

#. Array data is not shared by copying or slice indexing. Instead there
   is copy-on-write.

#. There are no zero or one-dimensional arrays. Scalars are
   two-dimensional rather than zero=dimensional as in numpy.

#. Single subscript implies ravel.

#. Boadcasting rules are different

Coexistence of matlab matrices and numpy arrays
===============================================

#. Empty vector::

        []                  matlabarray()

#. Scalars are 1x1 matrices::

        17                  [[ 17 ]]

#. Rectangular char arrays::

        'hello world'       char('hello world')

#. Row vector::

        [1 2 3]             [[1, 2, 3]]

#. Column vector::

        [1;2;3]             [[1], [2], [3]]

#. Cell strings::

        cellstr('abc','hello',[97 98 99])       


(*) Such expressions _are_ legal in Octave.  TBD

"""
from numpy import inf
import numpy as np
import os,sys,copy
from scipy.io import loadmat
import unittest

def isvector_or_scalar(a):
    """
    one-dimensional arrays having shape [N],
    row and column matrices having shape [1 N] and
    [N 1] correspondingly, and their generalizations
    having shape [1 1 ... N ... 1 1 1].
    Scalars have shape [1 1 ... 1].
    Empty arrays dont count
    """
    try:
        return a.size and a.ndim-a.shape.count(1) <= 1
    except:
        return False
def isvector(a):
    """
    one-dimensional arrays having shape [N],
    row and column matrices having shape [1 N] and
    [N 1] correspondingly, and their generalizations
    having shape [1 1 ... N ... 1 1 1]
    """
    try:
        return a.ndim-a.shape.count(1) == 1
    except:
        return False

class matlabarray(np.ndarray):
    """
    >>> matlabarray()
    matlabarray([], shape=(0, 0), dtype=float64)
    """

    def __new__(cls,a=[],dtype="float64"):
        obj = np.array(a,
                       dtype=dtype,
                       copy=False,
                       order="F",
                       ndmin=2).view(cls).copy(order="F")
        if obj.size == 0:
            obj.shape = (0,0)
        return obj

    #def __array_finalize__(self,obj):

    def __copy__(self):
        return np.ndarray.copy(self,order="F")

    def __iter__(self):
        """ must define iter or char won't work"""
        return np.asarray(self).__iter__()

    def compute_indices(self,index):
        if not isinstance(index,tuple):
           index = index,
        if len(index) != 1 and len(index) != self.ndim:
            raise IndexError
        indices = []
        for i,ix in enumerate(index):
            if ix.__class__ is slice:
                if self.size == 0 and ix.stop is None:
                    raise IndexError
                if len(index) == 1:
                    n = self.size
                else:
                    n = self.shape[i]
                indices.append(np.arange((ix.start or 1)-1,
                                          ix.stop  or n,
                                          ix.step  or 1,
                                          dtype=int))
            else:
                try:
                    indices.append(int(ix)-1)
                except:
                    indices.append(np.asarray(ix).astype("int32")-1)
        if len(indices) == 2 and isvector(indices[0]) and isvector(indices[1]):
            indices[0].shape = (-1,1)
            indices[1].shape = (-1,)
        return tuple(indices)

    def __getslice__(self,i,j):
        if i == 0 and j == sys.maxsize:
            return self.reshape(-1,1,order="F")
        return self.__getitem__(slice(i,j))

    def __getitem__(self,index):
        return matlabarray(self.get(index))

    def get(self,index):
        #import pdb; pdb.set_trace()
        indices = self.compute_indices(index)
        if len(indices) == 1:
            return np.ndarray.__getitem__(self.reshape(-1,order="F"),indices)
        else:
            return np.ndarray.__getitem__(self,indices)

    def __setslice__(self,i,j,value):
        if i == 0 and j == sys.maxsize:
            index = slice(None,None)
        else:
            index = slice(i,j)
        self.__setitem__(index,value)
        
    def sizeof(self,ix):
        if isinstance(ix,int):
            n = ix+1
        elif isinstance(ix,slice):
            n = ix.stop
        elif isinstance(ix,(list,np.ndarray)):
            n = max(ix)+1
        else:
            assert 0,ix
        if not isinstance(n,int):
            raise IndexError
        return n 

    def __setitem__(self,index,value):
        #import pdb; pdb.set_trace()
        indices = self.compute_indices(index)
        try:
            if len(indices) == 1:
                np.asarray(self).reshape(-1,order="F").__setitem__(indices,value)
            else:
                np.asarray(self).__setitem__(indices,value)
        except (ValueError,IndexError):
            #import pdb; pdb.set_trace()
            if not self.size:
                new_shape = [self.sizeof(s) for s in indices]
                self.resize(new_shape,refcheck=0)
                np.asarray(self).__setitem__(indices,value)
            elif len(indices) == 1:
                # One-dimensional resize is only implemented for
                # two cases:
                #
                # a. empty matrices having shape [0 0]. These
                #    matries may be resized to any shape.  A[B]=C
                #    where A=[], and B is specific -- A[1:10]=C
                #    rather than A[:]=C or A[1:end]=C
                if self.size and not isvector_or_scalar(self):
                    raise IndexError("One-dimensional resize "
                                     "works only on vectors, and "
                                     "row and column matrices")
                # One dimensional resize of scalars creates row matrices
                # ai = 3
                # a(4) = 1
                # 3 0 0 1
                n = self.sizeof(indices[0]) # zero-based
                if max(self.shape) == 1:
                    new_shape = list(self.shape)
                    new_shape[-1] = n
                else:
                    new_shape = [(1 if s==1 else n) for s in self.shape]
                self.resize(new_shape,refcheck=0)
                np.asarray(self).reshape(-1,order="F").__setitem__(indices,value)
            else:
                new_shape = list(self.shape)
                if self.flags["C_CONTIGUOUS"]:
                    new_shape[0] = self.sizeof(indices[0])
                elif self.flags["F_CONTIGUOUS"]:
                    new_shape[-1] = self.sizeof(indices[-1])
                self.resize(new_shape,refcheck=0)
                np.asarray(self).__setitem__(indices,value)

    def __repr__(self):
        return self.__class__.__name__ + repr(np.asarray(self))[5:]

    def __str__(self):
        return str(np.asarray(self))
 
    def __add__(self,other):
        return matlabarray(np.asarray(self)+np.asarray(other))

    def __neg__(self):
        return matlabarray(np.asarray(self).__neg__())

####
class cellarray(matlabarray):
    """
    Cell array corresponds to matlab ``{}``


    """

    def __new__(cls, a=[]):
        """
        Create a cell array and initialize it with a.
        Without arguments, create an empty cell array.

        Parameters:
        a : list, ndarray, matlabarray, etc.

        >>> a=cellarray([123,"hello"])
        >>> print a.shape
        (1, 2)

        >>> print a[1]
        123

        >>> print a[2]
        hello
        """
        obj = np.array(a,
                       dtype=object,
                       order="F",
                       ndmin=2).view(cls).copy(order="F")
        if obj.size == 0:
            obj.shape = (0,0)
        return obj

    def __getitem__(self,index): 
        return self.get(index)

#    def __str__(self):
#        if self.ndim == 0:
#            return ""
#        if self.ndim == 1:
#            return "".join(s for s in self)
#        if self.ndim == 2:
#            return "\n".join("".join(s) for s in self)
#        raise NotImplementedError


class cellstr(matlabarray):
    """
    >>> s=cellstr(char('helloworldkitty').reshape(3,5))
    >>> s
    cellstr([['hello', 'world', 'kitty']], dtype=object)
    >>> print s
    hello
    world
    kitty
    >>> s.shape
    (1, 3)
    """

    def __new__(cls, a):
        """
        Given a two-dimensional char object,
        create a cell array where each cell contains
        a line.
        """
        obj = np.array(["".join(s) for s in a], 
                       dtype=object,
                       copy=False,
                       order="C",
                       ndmin=2).view(cls).copy(order="F")
        if obj.size == 0:
            obj.shape = (0,0)
        return obj

    def __str__(self):
        return "\n".join("".join(s) for s in self.reshape(-1))

    def __getitem__(self,index): 
        return self.get(index)


class char(matlabarray):
    """
    class char is a rectangular string matrix, which
    inherits from matlabarray all its features except
    dtype.

    >>> s=char()
    >>> s.shape
    (0, 0)

    >>> s=char('helloworld').reshape(2,5)
    >>> print s
    hello
    world

    >>> s.shape
    (2, 5)
    """

    def __new__(cls, a=""):
        if not isinstance(a,str):
            raise NotImplementedError
        obj = np.array(list(a),
                       dtype='|S1',
                       copy=False,
                       order="F",
                       ndmin=2).view(cls).copy(order="F")
        if obj.size == 0:
            obj.shape = (0,0)
        return obj

    def __getitem__(self,index): 
        return self.get(index)

    def __str__(self):
        if self.ndim == 0:
            return ""
        if self.ndim == 1:
            return "".join(s for s in self)
        if self.ndim == 2:
            return "\n".join("".join(s) for s in self)
        raise NotImplementedError


def abs_(a):
    """
    Unless the argument is already as subclass of ndarray,
    convert the argument to ndarray, then apply numpy.abs
    """
    return np.abs(np.asanyarray(a))

def arange_(start,stop,step=1,**kwargs):
    """
    >>> a=arange_(1,10) # 1:10
    >>> size_(a)
    matlabarray([[  1.,  10.]])
    """
    return matlabarray(np.arange(start,
                                 stop+1,
                                 step,
                                 **kwargs).reshape(1,-1),**kwargs)

def ceil_(a):
    """
    Unless the argument is already as subclass of ndarray,
    convert the argument to ndarray, then apply numpy.ceil
    """
    return np.ceil(np.asanyarray(a))

def cell_(*args):
    if len(args) == 1:
        args += args
    return cellarray(np.zeros(args,dtype=object,order="F"))

def copy_(a):
    return matlabarray(np.asanyarray(a).copy(order="F"))

def disp_(*args):
    print (args)

false = False

def false_(*args):
    if len(args) == 1:
        args += args
    return np.zeros(args,dtype=bool,order="F")

def find_(a,n=None,d=None,nargout=1):
    if d:
        raise NotImplementedError

    # there is no promise that nonzero or flatnonzero
    # use or will use indexing of the argument without
    # converting it to array first.  So we use asarray
    # instead of asanyarray
    if nargout == 1:
        i = np.flatnonzero(np.asarray(a)).reshape(1,-1)+1
        if n is not None:
            i = i.take(n)
        return matlabarray(i)
    if nargout == 2:
        i,j = np.nonzero(np.asarray(a))
        if n is not None:
            i = i.take(n)
            j = j.take(n)
        return (matlabarray((i+1).reshape(-1,1)),
                matlabarray((j+1).reshape(-1,1)))
    raise NotImplementedError

def floor_(a):
    return np.floor_(np.asanyarray(a))

def fullfile_(*args):
    return os.path.join(*args)

def intersect_(a,b,nargout=1):
    if nargout == 1:
        c = sorted(set(a) & set(b))
        if isinstance(a,str):
            return "".join(c)
        elif isinstance(a,list):
            return c
        else:
            # FIXME: the result is a column vector if
            # both args are column vectors; otherwise row vector
            return np.array(c)
    raise NotImplementedError

#def inf_(*args):
#    t = np.empty(np.prod(args))
#    t[:] = np.inf
#    return t.reshape(args)
#
#Inf_ = inf_
#
#def int_(t):
#    return np.int(t)
#
#def int32_(t):
#    return np.int(t)
#

def iscellstr_(a):
    return isinstance(a,cellarray) and all(isinstance(t,str) for t in a)

def ischar_(a):
    return a.dtype == "|S1"

def isempty_(a):
    try:
        return 0 in np.asarray(a).shape
    except AttributeError:
        return False

def isequal_(a,b):
    return np.array_equal(np.asanyarray(a),
                          np.asanyarray(b))
                          

def length_(a):
    try:
        return max(np.asarray(a).shape)
    except ValueError:
        return 1

def load_(a):
    return loadmat(a) # FIXME

def max_(a, d=0, nargout=0):
    if d or nargout:
        raise NotImplementedError
    return np.amax(a)

def min_(a, d=0, nargout=0):
    if d or nargout:
        raise NotImplementedError
    return np.amin(a)

def ndims_(a):
    return np.asarray(a).ndim

def numel_(a):
    return np.asarray(a).size

def ones_(*args,**kwargs):
    if not args:
        return 1.0
    if len(args) == 1:
        args += args
    return matlabarray(np.ones(args,order="F",**kwargs))

def rand_(*args,**kwargs):
    if not args:
        return np.random.rand()
    if len(args) == 1:
        args += args
    try:
        return np.random.rand(np.prod(args)).reshape(args,order="F")
    except:
        pass

def ravel_(a):
    return np.asanyarray(a).reshape(-1,1)

def round_(a):
    return np.round(np.asanyarray(a))

def rows_(a):
    return np.asarray(a).shape[0]

def size_(a, b=0, nargout=1):
    """
    >>> size_(zeros_(3,3)) + 1
    matlabarray([[ 4.,  4.]])
    """
    s = np.asarray(a).shape
    if s is ():
        return 1 if b else (1,)*nargout
    # a is not a scalar
    try:
        if b:
            return s[b-1]
        else:
            return matlabarray(s) if nargout <= 1 else s
    except IndexError:
        return 1

def strread_(s, format="", nargout=1):
    if format == "":
        a = [float(x) for x in s.split()]
        return tuple(a) if nargout > 1 else np.asanyarray([a])
    raise ErrorNotImplemented

def strrep_(a,b,c):
    if isinstance(a,str):
        return a.replace(b,c)
    raise ErrorNotImplemented # cell arrays

def sum_(a, dim=None):
    if dim is None:
        return np.asanyarray(a).sum()
    else:
        return np.asanyarray(a).sum(dim-1)

true = True

def true_(*args):
    if len(args) == 1:
        args += args
    return matlabarray(np.ones(args,dtype=bool,order="F"))

def zeros_(*args,**kwargs):
    if not args:
        return 0.0
    if len(args) == 1:
        args += args
    return matlabarray(np.zeros(args,**kwargs))

if __name__ == "__main__":
    import doctest
    doctest.testmod()

# vim:et:sw=4:si:



import parse,sys
import networkx as nx
import node,resolve

def callgraph(func_list):
    """
    Build callgraph of func_list, ignoring
    built-in functions
    """
    G = nx.DiGraph()
    for func in func_list:
        G.add_node(func.head.ident.name)
    for func in func_list:
        assert isinstance(func,node.function)
        func_name = func.head.ident.name
        resolve.resolve(func)
        for s in node.postorder(func):
            if (s.__class__ is node.funcall and
                s.func_expr.__class__ is  node.ident and
                s.func_expr.name in G.nodes()):
                G.add_edge(func_name,s.func_expr.name)
    return G

G = nx.DiGraph()

def postorder_edge(u):
    if isinstance(u,node.node):
        for v in u:
            for t in postorder_edge(v):
                yield (v,t)
        yield (u,u) # returns only traversible objects

def foo(tree):
    G = nx.DiGraph()
    for u,v in postorder_edge(tree):
        G.add_edge(id(u),id(v))
    return G

def main():
    func_list = parse.parse(open(sys.argv[1]).read())
    G = foo(func_list)
    #G = callgraph(func_list)
    nx.write_dot(G,"G.dot")
    #H = nx.dfs_tree(G,'solver')
    #nx.write_dot(H,"H.dot")
    #print nx.topological_sort(H)

if __name__ == '__main__':
    main()



"""
 SMOP compiler -- Simple Matlab/Octave to Python compiler
 Copyright 2011-2013 Victor Leikehman
"""

import inspect
import sys
import node
from node import extend

import options,parse

ZERO = node.number(0)

def rewrite(t):
    global modified; modified = []
    while do_rewrite(t) or modified:
        modified = []

def do_rewrite(t):
    for u in node.postorder(t):
        try:
            u._rewrite()
        except:
            assert 0

def lineno():
    """Returns the current line number in our program."""
    return inspect.currentframe().f_back.f_lineno

@extend(node.node)
def _rewrite(self):
    pass

@extend(node.sum)
@extend(node.min)
@extend(node.max)
def _rewrite(self):
    """max(foo[:]) --> max(foo)
    """
    cls = self.__class__
    if (self.args[0].__class__ is node.arrayref
        and self.args[0].args[0].__class__ is node.expr
        and self.args[0].args[0].op == "::"
        and len(self.args[0].args[0].args) == 0):

        self.become(cls(self.args[0].func_expr))

# """
# @extend(node.false)
# @extend(node.inf)
# @extend(node.ones)
# @extend(node.true)
# @extend(node.zeros)
# def _rewrite(self):
#     if self.__class__ == node.false:
#         v = node.logical(0)
#     elif self.__class__ == node.true:
#         v = node.logical(1)
#     elif self.__class__ == node.zeros:
#         v = node.number(0)
#     elif self.__class__ == node.ones:
#         v = node.number(1)
#     elif self.__class__ == node.inf:
#         v = node.number(0) # FIXME FIXME FIXme
#     s = self._shape()
#     if not s:
#         self.become(v)
#         modified.append(lineno())
#         return
    
#     if options.row_vector_ndim == 1 and s and s[0].__class__ == node.number:
#         del s[0]

#     self.become(node.RESHAPE(node.matrix(v),
#                              node.matrix(*s),
#                              node.matrix(v)))
#     modified.append(lineno())



# # elemental monadic
# @extend(node.isnan)
# @extend(node.sign)
# def _rewrite(self):
#     cls = getattr(node,self.__class__.__name__.upper(),None)
#     assert issubclass(cls,node.builtins)
#     self.become(cls(self.args[0]))
#     modified.append(lineno())


# @extend(node.rand)
# def _rewrite(self):
#     # r = rand(n)
#     # r = rand(m,n)
#     # r = rand([m,n])
#     # r = rand(m,n,p,...)
#     # r = rand([m,n,p,...])
#     # r = rand
#     # r = rand(size(A))
#     # r = rand(..., 'double')
#     # r = rand(..., 'single')

#     cls = getattr(node,self.__class__.__name__.upper(),None)
#     assert issubclass(cls,node.builtins)
#     self.become(cls(*self.args))
#     modified.append(lineno())

# @extend(node.length)
# def _rewrite(self):
#     if not self.args[0]._shape():
#         self.become(ONE)
#         modified.append(lineno())
#     else:
#         self.become(node.MAXVAL(node.SHAPE(self.args[0])))
#         modified.append(lineno())

# @extend(node.ceil)
# def _rewrite(self):
#     self.become(node.CEILING(self.args[0]))
#     modified.append(lineno())

# @extend(node.mod)
# def _rewrite(self):
#     self.become(node.MODULO(self.args[0],self.args[1]))
#     modified.append(lineno())

# @extend(node.func_decl)
# def _rewrite(self):
#     for s in self.args:
#         if s.__class__ == node.ident:
#             s.__class__ = node.param
#             modified.append(lineno())

# # @extend(node.stmt_list)
# # @extend(node.global_list)
# # @extend(node.concat_list)
# # @extend(node.expr_list)
# # def _rewrite(self):
# #     for expr in self:
# #         expr._rewrite()

# class LET(node.let): # final fortran version of assignment
#     def _rewrite(self):
#         pass

# @extend(node.let)
# def _rewrite(self):
#     global debug_variables

#     if self.ret.__class__ == node.ident:
#         if self.ret.name in debug_variables:
#             pdb.set_trace()
#         try:
#             self.ret._t = self.args._type()
#             self.ret._r = self.args._rank()
#             self.ret._s = node.expr_list() if self.ret._r == 0 else self.args._shape()
#             assert isinstance(self.ret._s,node.expr_list)
#             #assert all([not d._shape() for d in self.ret._s])
#             for d in self.ret._s:
#                 assert not d._shape()

#             ndims = len(self.ret._s)
#             if ndims:
#                 #dim_list = node.expr_list([node.arrayref(self.ret, node.number(i+1)) for i in range(len(self.ret._s))]),
#                 new_self = node.stmt_list([node.allocate_stmt(self.ret, self.ret._s),
#                                            LET(self.ret,self.args)])
#                 logging.debug("%s ==> %s" % (self,new_self))
#                 self.become(new_self)
#                 modified.append(lineno())
#         except:
#             logging.exception("")
#             pass


# # @extend(node.func_decl)
# # def _rewrite(self):
# #     #We dont know a priori type and rank
# #     #of function arguments.  We do know that
# #     #to get their size we use SIZE(args[i])
# #     
# #     for a in self.args:
# #         if a.__class__ == node.ident:
# #             a.__class__ = node.param

# @extend(node.call_stmt)
# def _rewrite(self):
#     if (self.func_expr.__class__ == node.ident and
#         self.func_expr.name == "size" and
#         self.func_expr.defs == set()):
#         self.become(node.stmt_list([node.let(self.ret[0],node.SIZE(self.args,ONE)),
#                                     node.let(self.ret[1],node.SIZE(self.args,TWO))]))
#         modified.append(lineno())
#     elif (self.func_expr.__class__ == node.ident and
#         self.func_expr.name == "find" and
#         self.func_expr.defs == set()):
#         self.func_expr.name = "FIND"
#         self.ret[0]._s = node.expr_list()
#         self.ret[1]._s = node.expr_list()
#         modified.append(lineno())
        
# @extend(node.arrayref)
# def _rewrite(self):
#     pass
        

# @extend(node.funcall)
# def _rewrite(self):
#     if self.func_expr.__class__ == node.ident:
#         # Convert funcall nodes to array references.
#         if self.func_expr.defs:
#             self.__class__ = node.arrayref
#             modified.append(lineno())
#         elif self.func_expr.defs == set():
#             # Convert recognized builtin functions to builtin nodes.
#             cls = getattr(node,self.func_expr.name,None)
#             if cls and issubclass(cls,node.builtins) and self.__class__ != cls:
#                 logging.debug("%s ==> %s",self.__class__, cls)
#                 self.__class__ = cls
#                 modified.append(lineno())
#             else:
#                 pass
#                 #raise Exception("'%s' used but not defined" %
#                 #                self.func_expr.name)




# @extend(node.matrix)
# def _rewrite(self):
#     pass






import parse,sys
import node
from node import extend
import networkx as nx
 
                

def resolve(t,fp,func_name):
    fp.write("digraph %s {\n" % func_name)
    fp.write('graph [rankdir="LR"];\n')
    for u in node.postorder(t):
        if u.__class__ in (node.ident,node.param):
            fp.write("%s [label=%s_%s_%s];\n" % (u.lexpos,u.name,u.lineno,u.column))
            if u.defs:
                for v in u.defs:
                    fp.write("%s -> %s" % (u.lexpos,v.lexpos))
                    if u.lexpos < v.lexpos:
                        fp.write('[color=red]')
                    #else:
                    #    fp.write('[label=%s.%s]' % (v.lineno,v.column))
                    fp.write(';\n')
    fp.write("}\n")

@extend(node.node)
def _graphviz(self,fp):
    if getattr(self,"__slots__",False):
        fp.write('"%s" [' % id(self))
        fp.write('label = "<f0> %s|' % self.__class__.__name__)
        fp.write('|'.join(['<f%d> %s' % (i+1,s) for i,s in enumerate(self.__slots__)]))
        fp.write('"]\n')
    else:
        fp.write('"%s" [' % id(self))
        fp.write('label = "<f0> %s|' % self.__class__.__name__)
        fp.write('|'.join(['<f%d> %d' % (i+1,i+1) for i in range(len(self))]))
        fp.write('"]\n')

    for i,s in enumerate(self):
        if not isinstance(s,node.node):
            fp.write('"%s" [label="%s",shape=ellipse]\n' % (id(s),s))
        if not s in [None,0,"",False]:
            fp.write('"%s":f%d -> "%s";\n' % (id(self), i+1, id(s)))

@extend(node.number)
def _graphviz(self,fp):
    fp.write('"%s" [label="<f0> %s|<f1> %s"]\n' % 
             (id(self), self.__class__.__name__,self.value))

@extend(node.ident)
def _graphviz(self,fp):
    fp.write('"%s" [label="<f0> %s|<f1> %s"]\n' % 
             (id(self), self.__class__.__name__,self.name))

def graphviz(tree,fp):
    fp.write('''strict digraph g {
                graph [rankdir="LR"];
                node [shape=record];
             ''')
    for u in node.postorder(tree):
        u._graphviz(fp)
    fp.write("}\n")

if __name__ == '__main__':
    main()



import unittest
from runtime import *

class test_matlabarray(unittest.TestCase):
    """Expanding matlabarray"""
    def test010(self):
        """Two-dimensional assignment to []"""
        a = matlabarray()
        a[a.shape[0]+1,[1,2,3]] = [123,456,789]
        a[a.shape[0]+1,[1,2,3]] = [123,456,789]
        a[a.shape[0]+1,[1,2,3]] = [123,456,789]
        self.assertTrue(isequal_(a, [[123,456,789],
                                     [123,456,789],
                                     [123,456,789]]))
    def test020(self):
        """Two-dimensional assignment to []
           Expand, use list [1,2,3] for indexing"""
        a = matlabarray()
        a[a.shape[0]+1,[1,2,3]] = 123
        a[a.shape[0]+1,[1,2,3]] = 123
        a[a.shape[0]+1,[1,2,3]] = 123
        self.assertTrue(isequal_(a, [[123,123,123],
                                     [123,123,123],
                                     [123,123,123]]))

    def test030(self):
        """Two-dimensional assignment to []
           Expand, use slice 1:3 for indexing"""
        a = matlabarray()
        #import pdb; pdb.set_trace()
        a[a.shape[0]+1,1:3] = 123
        a[a.shape[0]+1,1:3] = 123
        a[a.shape[0]+1,1:3] = 123
        #print a.shape
        self.assertTrue(isequal_(a, [[123,123,123],
                                     [123,123,123],
                                     [123,123,123]]))
    #@unittest.skip("FIXME")
    def test040(self):
        a = matlabarray()
        with self.assertRaises(IndexError):
            a[a.shape[0]+1,:] = 123
            a[a.shape[0]+1,:] = 123
            a[a.shape[0]+1,:] = 123
            self.assertTrue(isequal_(a, [[123],
                                         [123],
                                         [123]]))
    @unittest.skip("wonders of matlab")
    def test050(self):
        """
        Compare to test060

        octave> a=[]
        a = []
        octave> a(:,:)=99
        a =  99
        octave> a
        a =  99
        octave> size(a)
        ans =

        1   1
        """
        a = matlabarray()
        a[:,:] = 99
        self.assertTrue(isequal_(a.item(0), 99))

    def test060(self):
        """One-dimensional assignment to empty array

        octave> a=[]
        a = []
        octave> a(:)=99
        a = []
        octave> a
        a = []
        """
        a = matlabarray()
        with self.assertRaises(IndexError):
            a[:] = 99
            self.assertTrue(isempty_(a))

    #@unittest.skip("wonders of matlab")
    def test062(self):
        """One-dimensional assignment to empty array

        octave> a=[]
        a = []
        octave> a(:)=[1 2 3]
        error: A(I) = X: X must have the same size as I
        """
        a = matlabarray()
        with self.assertRaises(Exception):
            a[:] = [1,2,3]

    #@unittest.skip("wonders of matlab")
    def test064(self):
        """One-dimensional assignment to empty array

        octave> a=[]
        a = []
        octave> a(1:3)=[1 2 3]
        1 2 3
        """
        a = matlabarray()
        a[1:3] = [1,2,3]
        self.assertTrue(isequal_(a, [1,2,3]))

    def test070(self):
        """
        wonders of matlab

        octave> c=[]
        c = []
        octave> c(1:end)=9
        c = []
        """
        a = matlabarray()
        a[1:a.shape[0]] = 9
        self.assertTrue(isempty_(a))

    @unittest.skip("wonders of matlab")
    def test080(self):
        """
        octave> a=[]
        a = []
        octave> a(1:end,5) = 5
        a = [](0x5)        % notice 0x5
        """

    def test084(self):
        """
        octave> a=[]
        a = []
        octave> a(:,5) = 5
        a =

        0   0   0   0   5
        """

    def test090(self):
        a = matlabarray([[11,22,33]])
        a[4] = 44
        self.assertTrue(isequal_(a,[[11,22,33,44]]))

    def test092(self):
        a = matlabarray([[11,22,33,44]])
        a[5:7] = [55,66,77]
        self.assertTrue(isequal_(a,[[11,22,33,44,55,66,77]]))

    def test094(self):
        a = matlabarray([[11,22,33,44,55,66,77]])
        a[[8,9]] = [88,99]
        self.assertTrue(isequal_(a,[[11,22,33,44,55,66,77,88,99]]))

    def test100(self):
        a = matlabarray([[1,3],
                         [2,4]])
        #a[: , a.shape[1]+1] = [5,6]
        a[: , 3] = [5,6]
        self.assertTrue(isequal_(a,[[1,3,5],
                                    [2,4,6]]))

    def test110(self):
        a = zeros_(4,4,dtype=int)
        a[2:3,2:3] = 1
        #print a
        self.assertTrue(isequal_(a,[[0,0,0,0],
                                    [0,1,1,0],
                                    [0,1,1,0],
                                    [0,0,0,0]]))
if __name__ == "__main__":
    unittest.main()
# vim:et:sw=4:si:



subscripts = "square" # "round"

"""Row vectors in Matlab can be expressed in Fortran as
   either one or two dimensional arrays"""
row_vector_ndim   = 1 # 2

"""Given Matlab code such as [a b c]=size(X) we heuristically
   decide that X is 3-dimensional."""
rank_guess_from_size = True

"""Given false(1,n), deduce that n is a scalar"""
rank_backward_propagate=1

"""0=not even constants
   1=char constants supported
"""
has_char_constants = 0

do_allocate = 0
do_resolve  = 1
do_rewrite  = 0
do_rename   = 0 # SSA
do_typeof   = 0
do_listing  = 0

debug = False

uppercase=True # active only with f90

line_numbering=True #True # uses either # or ! or %




# smop -- Matlab to Python compiler
# Copyright 2011-2013 Victor Leikehman
"""
if i.defs: 
    i is defined, possibly more than once.  
    Typical for vairable references.

if i.defs is None:
    i is a definition (lhs)
    
if i.defs == set():
    i is used but not defined.  
    Typical for function calls.

symtab is a temporary variable, which maps
variable names (strings) to sets of ident
instances, which possibly define the variable.
It is used in if_stmt, for_stmt, and while_stmt.
"""

import copy,sys,pprint

import node
from node import extend
import backend,options
import networkx as nx

def graphviz(t,fp,func_name):
    fp.write("digraph %s {\n" % func_name)
    fp.write('graph [rankdir="LR"];\n')
    for u in node.postorder(t):
        if u.__class__ in (node.ident,node.param):
            fp.write("%s [label=%s_%s_%s];\n" % (u.lexpos,u.name,u.lineno,u.column))
            if u.defs:
                for v in u.defs:
                    fp.write("%s -> %s" % (u.lexpos,v.lexpos))
                    if u.lexpos < v.lexpos:
                        fp.write('[color=red]')
                    #else:
                    #    fp.write('[label=%s.%s]' % (v.lineno,v.column))
                    fp.write(';\n')
    fp.write("}\n")

def as_networkx(t):
    G = nx.DiGraph()
    for u in node.postorder(t):
        if u.__class__ in (node.ident,node.param):
            uu = "%s_%s_%s" % (u.name,u.lineno,u.column)
            #label = "%s\\n%s" % (uu, u.props if u.props else "")
            G.add_node(uu, ident=u) # label=label)
            if u.defs:
                for v in u.defs:
                    vv = "%s_%s_%s" % (v.name,v.lineno,v.column)
                    G.add_node(vv, ident=v)
                    if u.lexpos < v.lexpos:
                        G.add_edge(uu,vv,color="red")
                    else:
                        G.add_edge(uu,vv,color="black")
    return G

def resolve(t, symtab=None, fp=None, func_name=None):
    if symtab is None:
        symtab = {}
    do_resolve(t,symtab)
    G = as_networkx(t)
    #import pdb;pdb.set_trace()
    for n in G.nodes():
        u = G.node[n]["ident"]
        if u.props:
            pass
        elif G.out_edges(n) and G.in_edges(n):
            u.props = "U" # upd
            #print u.name, u.lineno, u.column
        elif G.in_edges(n):
            u.props = "D" # def
        elif G.out_edges(n):
            u.props = "R" # ref
        else:
            u.props = "F" # ???
        G.node[n]["label"] = "%s\\n%s" % (n, u.props)

    for u in node.postorder(t):
        if u.__class__ is node.func_decl:
            u.ident.name += "_"
        elif u.__class__ is node.funcall:
            try:
                if u.func_expr.props in "UR": # upd,ref
                    u.__class__ = node.arrayref
                else:
                    u.func_expr.name += "_"
            except:
                pass

    for u in node.postorder(t):
        if u.__class__ in (node.arrayref,node.cellarrayref):
            for i,v in enumerate(u.args):
                if v.__class__ is node.expr and v.op == ":":
                    v.op = "::"
                for w in node.postorder(v):
                    if w.__class__ is node.expr and w.op == "end":
                        w.args[0] = u.func_expr
                        w.args[1] = node.number(i)

    for u in node.postorder(t):
        if u.__class__ is node.let:
            if (u.ret.__class__ is node.ident and
                u.args.__class__ is node.matrix):
                u.args = node.funcall(func_expr=node.ident("matlabarray"),
                                      args=node.expr_list([u.args]))

    H = nx.connected_components(G.to_undirected())
    for i,component in enumerate(H):
        for nodename in component:
            if G.node[nodename]["ident"].props == "R":
                has_update = 1
                break
        else:
            has_update = 0
        if has_update:
            for nodename in component:
                G.node[nodename]["ident"].props += "S"  # sparse
        #S = G.subgraph(nbunch)
        #print S.edges()
    return G

def do_resolve(t,symtab):
    t._resolve(symtab)

def copy_symtab(symtab):
    new_symtab = copy.copy(symtab)
    for k,v in new_symtab.items():
        new_symtab[k] = copy.copy(v)
    return new_symtab

@extend(node.function)
def _resolve(self,symtab):
    self.head._resolve(symtab)
    self.body._resolve(symtab)
    self.head.ret._resolve(symtab)
        
@extend(node.global_list)
@extend(node.concat_list)
@extend(node.expr_list)
def _resolve(self,symtab):
    for expr in self:
        expr._resolve(symtab)

@extend(node.global_list)
@extend(node.concat_list)
@extend(node.expr_list)
def _lhs_resolve(self,symtab):
    for expr in self:
        expr._lhs_resolve(symtab)
        
@extend(node.stmt_list)
def _resolve(self,symtab):
    for stmt in self:
        stmt._resolve(symtab)
        
@extend(node.number)
@extend(node.string)
def _resolve(self,symtab):
        pass

# @extend(node.call_stmt)
# def _resolve(self,symtab):
#     # TODO: does the order of A and B matter? Only if the
#     # evaluation of function args may change the value of the
#     # func_expr. 
#     self.func_expr._resolve(symtab) # A
#     self.args._resolve(symtab)      # B
#     self.ret._lhs_resolve(symtab)

@extend(node.let)
def _lhs_resolve(self,symtab):
    self.args._resolve(symtab)
    self.ret._lhs_resolve(symtab)

@extend(node.let)
def _resolve(self,symtab):
    self.args._resolve(symtab)
    self.ret._lhs_resolve(symtab)

@extend(node.func_decl)
def _resolve(self,symtab):
    if self.ident:
        self.ident._resolve(symtab)
    self.args._lhs_resolve(symtab)
    self.ret._resolve(symtab)

@extend(node.for_stmt)
def _resolve(self,symtab):
    symtab_copy = copy_symtab(symtab)
    self.ident._lhs_resolve(symtab)
    self.expr._resolve(symtab)
    self.stmt_list._resolve(symtab)
    self.stmt_list._resolve(symtab) # 2nd time, intentionally
    # Handle the case where FOR loop is not executed
    for k,v in symtab_copy.items():
        symtab.setdefault(k,set()).update(v)

@extend(node.if_stmt)
def _resolve(self,symtab):
    symtab_copy = copy_symtab(symtab)
    self.cond_expr._resolve(symtab)
    self.then_stmt._resolve(symtab)
    if self.else_stmt:
        self.else_stmt._resolve(symtab_copy)
    for k,v in symtab_copy.items():
        symtab.setdefault(k,set()).update(v)
        
@extend(node.continue_stmt)  # FIXME
@extend(node.break_stmt)     # FIXME
def _resolve(self,symtab):
    pass

@extend(node.global_stmt)
def _resolve(self,symtab):
    self.global_list._lhs_resolve(symtab)
        
@extend(node.return_stmt)
def _resolve(self,symtab):
    self.ret._resolve(symtab)
        #symtab.clear()

@extend(node.expr_stmt)
def _resolve(self,symtab):
    self.expr._resolve(symtab)
        
@extend(node.where_stmt) # FIXME where_stmt ???
@extend(node.while_stmt)
def _resolve(self,symtab):
    symtab_copy = copy_symtab(symtab)
    self.cond_expr._resolve(symtab)
    self.stmt_list._resolve(symtab)
    self.cond_expr._resolve(symtab)
    self.stmt_list._resolve(symtab)
    # Handle the case where WHILE loop is not executed
    for k,v in symtab_copy.items():
        symtab.setdefault(k,set()).update(v)

@extend(node.try_catch)
def _resolve(self,symtab):
    self.try_stmt._resolve(symtab)
    self.catch_stmt._resolve(symtab) # ???

@extend(node.ident)
def _lhs_resolve(self,symtab):
    symtab[self.name] = set([self])
 
@extend(node.ident)
def _resolve(self,symtab):
    if self.defs is None:
        self.defs = set()
    try:
        self.defs |= symtab[self.name]
    except KeyError:
        # defs == set() means name used, but not defined
        pass
        
@extend(node.arrayref)
@extend(node.cellarrayref)
@extend(node.funcall)
def _resolve(self,symtab):
    # Matlab does not allow foo(bar)(bzz), so func_expr is usually
    # an ident, though it may be a field or a dot expression.
    if self.func_expr:
        self.func_expr._resolve(symtab)
    self.args._resolve(symtab)
    #if self.ret:
    #    self.ret._lhs_resolve(symtab)

@extend(node.setfield) # a subclass of funcall
def _resolve(self,symtab):
    self.func_expr._resolve(symtab)
    self.args._resolve(symtab)
    self.args[0]._lhs_resolve(symtab) 

@extend(node.arrayref)
@extend(node.cellarrayref)
@extend(node.funcall)    
def _lhs_resolve(self,symtab):
    # Definitely lhs array indexing.  It's both a ref and a def.
    # Must properly handle cases such as foo(foo(17))=42
    # Does the order of A and B matter?
    self.func_expr._resolve(symtab) # A
    self.args._resolve(symtab)      # B
    self.func_expr._lhs_resolve(symtab)
        
@extend(node.expr)
def _resolve(self,symtab):
    for expr in self.args:
        expr._resolve(symtab)

@extend(node.expr)
def _lhs_resolve(self,symtab):
    if self.op == ".": # see setfield
        self.args._resolve(symtab)
        self.args[0]._lhs_resolve(symtab)
    elif self.op == "[]":
        for arg in self.args:
            arg._lhs_resolve(symtab)

        




# SMOP compiler -- Simple Matlab/Octave to Python compiler
# Copyright 2011-2014 Victor Leikehman

import copy
import pprint
import os
import operator
import sys

import re
import yacc

from lexer import tokens
import lexer

#import builtins
import node
#from node import *
import resolve

# ident properties (set in parse.py)
# ----------------------------------
# G global
# P persistent
# A function argument
# F function return value
# I for-loop iteration index
# 
# ident properties (set in resolve.py)
# ------------------------------------
# R ref    =...a  or  =...a(b) 
# D def    a=...  or   [a,b,c]=...
# U update a(b)=...  or  [a(b) c(d)]=...

class error(Exception):
    pass

class syntax_error(error):
    pass

precedence = (
    ("right","DOTMULEQ","EQ","MULEQ","MINUSEQ","DIVEQ","PLUSEQ","OREQ","ANDEQ"),
    ("nonassoc","HANDLE"),
    ("left", "COMMA"),
    ("left", "COLON"),
    ("left", "ANDAND", "OROR"),
    ("left", "EQEQ", "NE", "GE", "LE", "GT", "LT"),
    ("left", "OR", "AND"),
    ("left", "PLUS", "MINUS"),
    ("left", "MUL","DIV","DOTMUL","DOTDIV","BACKSLASH"),
    ("right","UMINUS","NEG"),
    ("right","TRANSPOSE"),
    ("right","EXP", "DOTEXP"),
    ("nonassoc","LPAREN","RPAREN","RBRACE","LBRACE"),
    ("left", "FIELD","DOT","PLUSPLUS","MINUSMINUS"),
    )

def p_top(p):
    """
    top :
        | stmt_list
        | top func_decl stmt_list_opt
        | top func_decl stmt_list END_STMT semi_opt
    """
    if len(p) == 1:
        p[0] = node.stmt_list()
    elif len(p) == 2:
        p[0] = p[1]
    else:
        # we backpatch the func_decl node
        assert p[2].__class__ is node.func_decl
        p[2].use_nargin = use_nargin

        try:
            if p[3][-1].__class__ is not node.return_stmt:
                p[3].append(node.return_stmt(ret_expr))
        except:
            raise syntax_error(p)

        p[0] = p[1]
        p[0].append(node.function(head=p[2],body=p[3]))
    assert isinstance(p[0],node.stmt_list)

def p_semi_opt(p):
    """
    semi_opt :
             | semi_opt SEMI
             | semi_opt COMMA
    """
    pass

def p_stmt(p):
    """
    stmt : continue_stmt
         | break_stmt
         | expr_stmt
         | global_stmt
         | persistent_stmt
         | command
         | for_stmt
         | if_stmt
         | null_stmt
         | return_stmt
         | switch_stmt
         | try_catch
         | while_stmt
         | unwind
    """
    # END_STMT is intentionally left out
    p[0] = p[1]

def p_unwind(p):
    """
    unwind : UNWIND_PROTECT stmt_list UNWIND_PROTECT_CLEANUP stmt_list END_UNWIND_PROTECT
    """
    p[0] = node.try_catch(try_stmt=p[2],
                          catch_stmt=node.expr_list(),
                          finally_stmt=p[4])

def p_arg1(p):
    """
    arg1 : STRING
         | NUMBER
         | IDENT
         | GLOBAL
    """
    # a hack to support "clear global"
    p[0] = node.string(value=str(p[1]),
                       lineno=p.lineno(1),
                       lexpos=p.lexpos(1))

def p_args(p):
    """
    args : arg1
         | args arg1
    """
    if len(p) == 2:
        p[0] = node.expr_list([p[1]])
    else:
        p[0] = p[1]
        p[0].append(p[2])

def p_command(p):
    """
    command : ident args SEMI
    """
#    if p[1].name == "load":
#        # "load filename x" ==> "x=load(filename)"
#        # "load filename x y z" ==> "(x,y,z)=load(filename)"
#        ret=node.expr_list([node.ident(t.value) for t in p[2][1:]])
#        p[0] = node.funcall(func_expr=p[1],
#                            args=node.expr_list(p[2]),
#                            ret=ret)
#    else:
    p[0] = node.funcall(p[1],p[2])

####################
def p_global_list(p):
    """global_list : ident
                   | global_list ident
    """
    if len(p) == 2:
        p[0] = node.global_list([p[1]])
    elif len(p) == 3:
        p[0] = p[1]
        p[0].append(p[2])

def p_global_stmt(p):
    "global_stmt : GLOBAL global_list SEMI"
    p[0] = node.global_stmt(p[2])
    for ident in p[0]:
        ident.props="G"  # G=global

def p_persistent_stmt(p):
    #"""persistent_stmt :  PERSISTENT global_list SEMI
    """persistent_stmt :  PERSISTENT expr SEMI
    """
#    if len(p) == 4:
#        p[0] = node.global_stmt(p[2])
#        for ident in p[0]:
#            ident.props="G"  # G=global
#    else:
    assert p[2].__class__ in (node.let,node.ident), p[2].__class__
    p[0] = p[2]
    #print p[2]


def p_return_stmt(p):
    "return_stmt : RETURN SEMI"
    p[0] = node.return_stmt(ret=ret_expr)

def p_continue_stmt(p):
    "continue_stmt : CONTINUE SEMI"
    p[0] = node.continue_stmt(None)

def p_break_stmt(p):
    "break_stmt : BREAK SEMI"
    p[0] = node.break_stmt(None)

# switch-case-otherwise

def p_switch_stmt(p):
    """
    switch_stmt : SWITCH expr semi_opt case_list END_STMT
    """
    def backpatch(expr,stmt):
        if isinstance(stmt,node.if_stmt):
            stmt.cond_expr.args[1] = expr
            backpatch(expr,stmt.else_stmt)
    backpatch(p[2],p[4])
    p[0] = p[4]

def p_case_list(p):
    """
    case_list : 
              | CASE expr sep stmt_list_opt case_list
              | OTHERWISE stmt_list
    """
    if len(p) == 1:
        p[0] = node.stmt_list()
    elif len(p) == 3:
        assert isinstance(p[2],node.stmt_list)
        p[0] = p[2]
    elif len(p) == 6:
        p[0] = node.if_stmt(cond_expr=node.expr(op="==",
                                                args=node.expr_list([p[2]])),
                            then_stmt=p[4],
                            else_stmt=p[5])
        p[0].cond_expr.args.append(None) # None will be replaced using backpatch()
    else:
        assert 0

# try-catch

def p_try_catch(p):
    """
    try_catch : TRY stmt_list CATCH stmt_list END_STMT
              | TRY stmt_list END_STMT
    """
    assert isinstance(p[2],node.stmt_list)
    #assert isinstance(p[4],node.stmt_list)
    p[0] = node.try_catch(try_stmt=p[2],
                          catch_stmt=node.stmt_list(), # FIXME
                          finally_stmt=node.stmt_list())

def p_null_stmt(p):
    """
    null_stmt : SEMI
              | COMMA
    """
    p[0] = None
    
def p_func_decl(p):
    """func_decl : FUNCTION ident args_opt SEMI 
                 | FUNCTION ret EQ ident args_opt SEMI 
    """
    global ret_expr,use_nargin
    use_nargin = 0

    if len(p) == 5:
        assert isinstance(p[3],node.expr_list)
        p[0] = node.func_decl(ident=p[2],
                              ret=node.expr_list(),
                              args=p[3])
        ret_expr = node.expr_list()
    elif len(p) == 7:
        assert isinstance(p[2],node.expr_list)
        assert isinstance(p[5],node.expr_list)
        p[0] = node.func_decl(ident=p[4],
                              ret=p[2],
                              args=p[5])
        ret_expr = p[2]
    else:
        assert 0

def p_args_opt(p):
    """
    args_opt :
             | LPAREN RPAREN
             | LPAREN expr_list RPAREN
    """
    if len(p) == 1:
        p[0] = node.expr_list()
    elif len(p) == 3:
        p[0] = node.expr_list()
    elif len(p) == 4:
        assert isinstance(p[2],node.expr_list)
        p[0] = p[2]
    else:
        assert 0

def p_arg_list(p):
    """
    arg_list : ident
             | arg_list COMMA ident
    """
    if len(p) == 2:
        p[1].__class__ = node.param
        p[0] = node.expr_list([p[1]])
    elif len(p) == 4:
        p[0] = p[1]
        p[3].__class__ = node.param
        p[0].append(p[3])
    else:
        assert 0
    assert isinstance(p[0],node.expr_list)
    for ident in p[0]:
        ident.props="A"

def p_ret(p):
    """
    ret : ident
        | LBRACKET RBRACKET
        | LBRACKET expr_list RBRACKET
    """
    if len(p) == 2:
        p[0] = node.expr_list([p[1]])
    elif len(p) == 3:
        p[0] = node.expr_list([])
    elif len(p) == 4:
        assert isinstance(p[2],node.expr_list)
        p[0] = p[2]
    else:
        assert 0
    for ident in p[0]:
        ident.props="F"
# end func_decl

def p_stmt_list_opt(p):
    """
    stmt_list_opt : 
                  | stmt_list
    """
    if len(p) == 1:
        p[0] = node.stmt_list()
    else:
        p[0] = p[1]

def p_stmt_list(p):
    """
    stmt_list : stmt
              | stmt_list stmt
    """
    if len(p) == 2:
        p[0] = node.stmt_list([p[1]] if p[1] else [])
    elif len(p) == 3:
        p[0] = p[1]
        if p[2]:
            p[0].append(p[2])
    else:
        assert 0

def p_concat_list(p):
    """
    concat_list : expr_list SEMI expr_list
                | concat_list SEMI expr_list
    """
    if p[1].__class__ == node.expr_list:
        p[0] = node.concat_list([p[1],p[3]])
    else:
        p[0] = p[1]
        p[0].append(p[3])

def p_expr_list(p):
    """
    expr_list : exprs
              | exprs COMMA
    """
    p[0] = p[1]

def p_exprs(p):
    """
    exprs : expr
          | exprs COMMA expr
    """
    if len(p) == 2:
        p[0] = node.expr_list([p[1]])
    elif len(p) == 4:
        p[0] = p[1]
        p[0].append(p[3])
    else:
        assert(0)
    assert isinstance(p[0],node.expr_list)

def p_expr_stmt(p):
    """
    expr_stmt : expr_list SEMI
    """
    assert isinstance(p[1],node.expr_list)
    p[0] = node.expr_stmt(expr=p[1])

def p_while_stmt(p):
    """
    while_stmt : WHILE expr SEMI stmt_list END_STMT
    """
    assert isinstance(p[4],node.stmt_list)
    p[0] = node.while_stmt(cond_expr=p[2],
                           stmt_list=p[4])

def p_separator(p):
    """
    sep : COMMA
        | SEMI
    """
    p[0] = p[1]

def p_if_stmt(p):
    """
    if_stmt : IF expr sep stmt_list_opt elseif_stmt END_STMT
            | IF expr error stmt_list_opt elseif_stmt END_STMT
    """
    p[0] = node.if_stmt(cond_expr=p[2],
                        then_stmt=p[4],
                        else_stmt=p[5])

def p_elseif_stmt(p):
    """
    elseif_stmt :
                | ELSE stmt_list_opt
                | ELSEIF expr sep stmt_list_opt elseif_stmt 
    """
    if len(p) == 1:
        p[0] = node.stmt_list()
    elif len(p) == 3:
        p[0] = p[2]
    elif len(p) == 6:
        p[0] = node.if_stmt(cond_expr=p[2],
                            then_stmt=p[4],
                            else_stmt=p[5])
    else:
        assert 0


def p_for_stmt(p):
    """
    for_stmt : FOR ident EQ expr SEMI stmt_list END_STMT
             | FOR LPAREN ident EQ expr RPAREN SEMI stmt_list END_STMT
             | FOR matrix EQ expr SEMI stmt_list END_STMT
    """
    if len(p) == 8:
        if not isinstance(p[2],node.ident):
            raise NotImplementedError("for loop")
        p[2].props="I" # I= for-loop iteration variable
        p[0] = node.for_stmt(ident=p[2],
                             expr=p[4],
                             stmt_list=p[6])
################  expr  ################

def p_expr(p):
    """expr : ident
            | end
            | number
            | string
            | colon
            | NEG
            | matrix
            | cellarray
            | expr2
            | expr1
            | lambda_expr
            | ident PLUSPLUS
            | ident MINUSMINUS
    """
    #        | PLUSPLUS ident
    #        | MINUSMINUS ident
    if p[1]=="~":
        p[0] = node.ident(name="__")
    else:
        p[0] = p[1]

def p_lambda_args(p):
    """lambda_args : LPAREN RPAREN
                   | LPAREN arg_list RPAREN
    """
    p[0] = p[2] if len(p) == 4 else node.expr_list()

def p_lambda_expr(p):
    """lambda_expr : HANDLE lambda_args expr
    """
    p[0] = node.lambda_expr(args=p[2], ret=p[3])

def p_expr_ident(p):
    "ident : IDENT"
    if p[1] == "nargin":
        global use_nargin
        use_nargin += 1
    #import pdb; pdb.set_trace()
    p[0] = node.ident(name=p[1],
                      lineno=p.lineno(1),
                      lexpos=p.lexpos(1),
                      column=p.lexpos(1) - p.lexer.lexdata.rfind("\n",0,p.lexpos(1)))

def p_expr_number(p):
    "number : NUMBER"
    p[0] = node.number(p[1],lineno=p.lineno(1),lexpos=p.lexpos(1))

def p_expr_end(p):
    "end : END_EXPR"
    p[0] = node.expr(op="end",args=node.expr_list([node.number(0),
                                                   node.number(0)]))
    
def p_expr_string(p):
    "string : STRING"
    p[0] = node.string(p[1],lineno=p.lineno(1),lexpos=p.lexpos(1))

def p_expr_colon(p):
    "colon : COLON"
    p[0] = node.expr(op=":",args=node.expr_list())
    
def p_expr1(p):
    """expr1 : MINUS expr %prec UMINUS
             | PLUS expr %prec UMINUS
             | NEG expr
             | HANDLE ident
             | PLUSPLUS ident
             | MINUSMINUS ident
    """
    p[0] = node.expr(op=p[1],args=node.expr_list([p[2]]))

def p_cellarray(p):
    """
    cellarray : LBRACE RBRACE
              | LBRACE expr_list RBRACE
    """
    if len(p) == 3:
        p[0] = node.cellarray(op="{}",args=node.expr_list())
    else:
        p[0] = node.cellarray(op="{}",args=p[2])
    
def p_matrix(p):
    """matrix : LBRACKET RBRACKET
              | LBRACKET concat_list RBRACKET
              | LBRACKET concat_list SEMI RBRACKET
              | LBRACKET expr_list RBRACKET
              | LBRACKET expr_list SEMI RBRACKET
    """
    if len(p) == 3:
        p[0] = node.matrix()
    else:
        p[0] = node.matrix(p[2])
    
def p_paren_expr(p):
    """
    expr :  LPAREN expr RPAREN
    """
    p[0] = node.expr(op="parens",args=node.expr_list([p[2]]))

def p_field_expr(p):
    """
    expr : expr FIELD 
    """
    p[0] = node.expr(op=".",
                     args=node.expr_list([p[1],
                                          node.ident(name=p[2],
                                                     lineno=p.lineno(2),
                                                     lexpos=p.lexpos(2))]))

def p_transpose_expr(p):
    # p[2] contains the exact combination of plain and conjugate
    # transpose operators, such as "'.''.''''".
    "expr : expr TRANSPOSE"
    p[0] = node.transpose(p[1],node.string(p[2]))

def p_cellarrayref(p):
    """expr : expr LBRACE expr_list RBRACE
            | expr LBRACE RBRACE
    """
    args = node.expr_list() if len(p) == 4 else p[3]
    assert isinstance(args,node.expr_list)
    p[0] = node.cellarrayref(func_expr=p[1],args=args)

def p_funcall_expr(p):
    """expr : expr LPAREN expr_list RPAREN 
            | expr LPAREN RPAREN
    """
    if (0 and len(p)==5 and
        len(p[3])==1 and 
        p[3][0].__class__ is node.expr and
        p[3][0].op == ":" and not p[3][0].args):
        # foo(:) => ravel(foo)
        p[0] = node.funcall(func_expr=node.ident("ravel"),
                            args=node.expr_list([p[1]]))
    else:
        args = node.expr_list() if len(p) == 4 else p[3]
        assert isinstance(args,node.expr_list)
        p[0] = node.funcall(func_expr=p[1],args=args)
    
def p_expr2(p):
    """expr2 : expr AND expr
             | expr ANDAND expr
             | expr BACKSLASH expr
             | expr COLON expr
             | expr DIV expr
             | expr DOT expr
             | expr DOTDIV expr
             | expr DOTEXP expr
             | expr DOTMUL expr
             | expr DOTMULEQ expr
             | expr EQEQ expr
             | expr EXP expr
             | expr GE expr
             | expr GT expr 
             | expr LE expr
             | expr LT expr
             | expr MINUS expr
             | expr MUL expr
             | expr NE expr
             | expr OR expr
             | expr OROR expr
             | expr PLUS expr
             | expr EQ expr
             | expr MULEQ expr
             | expr DIVEQ expr
             | expr MINUSEQ expr
             | expr PLUSEQ expr
             | expr OREQ expr
             | expr ANDEQ expr
    """
    if p[2] == "=":
        if p[1].__class__ is node.let:
            raise NotImplementedError("assignment "
                                      "as expression")

        # The algorithm, which decides if an
        # expression F(X)
        # is arrayref or funcall, is implemented in
        # resolve.py, except the following lines up
        # to XXX. These lines handle the case where
        # an undefined array is updated:
        #    >>> clear a
        #    >>> a[1:10]=123
        # Though legal in matlab, these lines
        # confuse the algorithm, which thinks that
        # the undefined variable is a function name.
        # To prevent the confusion, we mark these
        # nodes arrayref as early as during the parse
        # phase.
        if p[1].__class__ is node.funcall:
            # A(B) = C
            p[1].__class__ = node.arrayref
        elif p[1].__class__ is node.matrix:
            # [A1(B1) A2(B2) ...] = C
            for e in p[1].args:
                if e.__class__ is node.funcall:
                    e.__class__ = node.arrayref
        # XXX   

        if isinstance(p[1],node.getfield):
            #import pdb;pdb.set_trace()
            # A.B=C  setfield(A,B,C)
            p[0] = node.setfield(p[1].args[0],
                                 p[1].args[1],
                                 p[3])
        else:
            #assert len(p[1].args) > 0
            ret = p[1].args if isinstance(p[1],node.matrix) else p[1]
            p[0] = node.let(ret=ret,
                            args=p[3],
                            lineno=p.lineno(2),
                            lexpos=p.lexpos(2))

            if isinstance(p[1],node.matrix):
                # TBD: mark idents as "P" - persistent
                assert p[3].__class__ in (node.ident,node.funcall), p[3].__class__
                if p[3].__class__ is node.ident:
                    # [A1(B1) A2(B2) ...] = F     implied F()
                    # import pdb; pdb.set_trace()
                    p[3] = node.funcall(func_expr=p[3],
                                        args=node.expr_list())
                # [A1(B1) A2(B2) ...] = F(X)
                p[3].nargout = len(p[1].args[0])
    elif p[2] == ".*":
        p[0] = node.dot(p[1],p[3])
    elif p[2] == "." and isinstance(p[3],node.expr) and p[3].op=="parens":
        p[0] = node.getfield(p[1],p[3].args[0])
        raise NotImplementedError(p[3],p.lineno(3),p.lexpos(3))
    elif p[2] == ":" and isinstance(p[1],node.expr) and p[1].op==":":
        # Colon expression means different things depending on the
        # context.  As an array subscript, it is a slice; otherwise,
        # it is a call to the "range" function, and the parser can't
        # tell which is which.  So understanding of colon expressions
        # is put off until after "resolve".
        p[0] = p[1]
        p[0].args.insert(1,p[3])
    else:
        p[0] = node.expr(op=p[2],args=node.expr_list([p[1],p[3]]))

opt_exclude = []
def p_error(p):
    if p is None:
        if p not in opt_exclude:
            raise syntax_error(p)
    elif p.lexpos not in opt_exclude:
        raise syntax_error(p)
    #print "Ignored:",p
    return p

parser = yacc.yacc(start="top")

def parse(buf,filename=""):
    try:
        new_lexer = lexer.new()
        p = parser.parse(buf,tracking=1,debug=0,lexer=new_lexer)
        return p
    except lexer.IllegalCharacterError as (lineno,column,c):
        #import pdb; pdb.set_trace()
        print 'Error:%s:%s.%s:illegal character:%s' % (filename,lineno,column,c)
        return []
    except NotImplementedError as e:
        print 'Error:%s:%s:not implemented:%s' % (filename,e[1],e[0])
        return []
    except syntax_error as e:
        try:
            #import pdb;pdb.set_trace()
            column=e[0].lexpos - new_lexer.lexdata.rfind("\n",0,e[0].lexpos)
            print '%s:%s.%s:syntax error %s=<%s>' % (filename,
                                                     e[0].lineno,
                                                     column,
                                                     e[0].type,
                                                     e[0].value)
        except:
            print "%s:unexpected EOF" % filename
        return []

# def fparse(filename):
#     buf = open(filename).read()
#     return parse(buf)

# vim: ts=4:sw=4:et:si:ai



# SMOP compiler -- Simple Matlab/Octave to Python compiler
# Copyright 2011-2014 Victor Leikehman

import sys
import re
from zlib import adler32
import lex
from lex import TOKEN
import readline

class IllegalCharacterError(Exception):
    pass

tokens = [
    "AND", "ANDAND", "ANDEQ", "BACKSLASH", "COLON", "COMMA", "DIV","DIVEQ",
    "DOT", "DOTDIV", "DOTEXP", "DOTMUL","DOTMULEQ", "END_EXPR",
    "END_STMT", "EQ", "EQEQ", "EXP", "FIELD", "GE", "GT", "HANDLE",
    "IDENT", "LBRACE", "LBRACKET", "LE", "LPAREN", "LT",
    "MINUS","MINUSMINUS","MINUSEQ","MUL","MULEQ","NE", "NEG",
    "NUMBER", "OR","OREQ", "OROR", "PLUS", "PLUSEQ","PLUSPLUS",
    "RBRACE", "RBRACKET", "RPAREN", "SEMI", "STRING", "TRANSPOSE",
]

reserved = {
    "break"                  : "BREAK",
    "case"                   : "CASE",
    "catch"                  : "CATCH",
    "continue"               : "CONTINUE",
    "else"                   : "ELSE",
    "elseif"                 : "ELSEIF",
    "end_unwind_protect"     : "END_UNWIND_PROTECT",
    "for"                    : "FOR",
    "function"               : "FUNCTION",
    "global"                 : "GLOBAL",
    "if"                     : "IF",
    "otherwise"              : "OTHERWISE",
    "persistent"             : "PERSISTENT",
    "return"                 : "RETURN",
    "switch"                 : "SWITCH",
    "try"                    : "TRY",
    "unwind_protect"         : "UNWIND_PROTECT",
    "unwind_protect_cleanup" : "UNWIND_PROTECT_CLEANUP",
    "while"                  : "WHILE",
    }
tokens += list(reserved.values())
#literals = "="

def new():
    t_AND         = r"\&"
    t_ANDAND      = r"\&\&"
    t_ANDEQ       = r"\&="
    t_BACKSLASH   = r"\\"
    t_COLON       = r":"
    t_DIV         = r"\/"
    t_DIVEQ       = r"\/="
    t_DOT         = r"\."
    t_DOTDIV      = r"\./"
    t_DOTEXP      = r"\.\^"
    t_DOTMUL      = r"\.\*"
    t_DOTMULEQ    = r"\.\*="
    t_EQ          = r"="
    t_EQEQ        = r"=="
    t_EXP         = r"\^"
    t_GE          = r">="
    t_GT          = r"\>"
    t_HANDLE      = r"\@"
    t_LE          = r"<="
    t_LT          = r"\<"
    t_MINUS       = r"\-"
    t_MINUSEQ     = r"\-="
    t_MINUSMINUS  = r"\--"
    t_MUL         = r"\*"
    t_MULEQ       = r"\*="
    t_NE          = r"(~=)|(!=)"
    t_NEG         = r"\~|!"
    t_OR          = r"\|"
    t_OREQ        = r"\|="
    t_OROR        = r"\|\|"
    t_PLUS        = r"\+"
    t_PLUSEQ      = r"\+="
    t_PLUSPLUS    = r"\+\+"
    
    states = (("matrix","inclusive"),
              ("afterkeyword","exclusive"))

    ws  = r"(\s|(\#|%).*\n|\.\.\..*\n|\\\n)"
    ws1 = ws+"+"
    ws0 = ws+"*"
    ms  = r"'([^']|(''))*'" 
    os  = r'"([^"\a\b\r\t\0\v\n\\]|(\\[abn0vtr\"\n\\]))*"'
    mos = "(%s)|(%s)" % (os,ms)
    id  = r"[a-zA-Z_][a-zA-Z_0-9]*"
    
    def unescape(s):
        if s[0] == "'":
            return s[1:-1].replace("''","'")
        else:
            return s[1:-1].decode('string_escape')
#            return s[1:-1].replace('\\"','"').replace('\\\n','').replace('\\n','\n').replace('\\\\','\\').replace('\\0','\0').replace('\\v','\v').replace('\\t','\t').replace('\\r','\r')

    @TOKEN(mos)
    def t_afterkeyword_STRING(t):
        t.value = unescape(t.value)
        t.lexer.begin("INITIAL")
        return t

    def t_afterkeyword_error(t):
        raise SyntaxError

    # A quote, immediately following any of: (1) an alphanumeric
    # charater, (2) right bracket, parenthesis or brace,
    # or (3) another TRANSPOSE, is a TRANSPOSE.  Otherwise, it starts a
    # string.  The order of the rules for TRANSPOSE (first) and STRING
    # (second) is important.  Luckily, if the quote is separated from
    # the term by line continuation (...), matlab starts a string, so
    # the above rule still holds.

    def t_TRANSPOSE(t):
        r"(?<=\w|\]|\)|\})((\.')|')+"
        # <---context ---><-quotes->
        # We let the parser figure out what that mix of quotes and
        # dot-quotes, which is kept in t.value, really means.
        return t

    @TOKEN(mos)
    def t_STRING(t):
        t.value = unescape(t.value)
        return t

    @TOKEN(r"(\.%s)?%s" % (ws0,id))
    def t_IDENT(t):
        t.lexer.lineno += t.value.count("\n")
        if t.value[0] == ".":
            # Reserved words are not reserved when used as fields.
            # So return=1 is illegal, but foo.return=1 is fine.
            t.type = "FIELD"
            return t
        if t.value in ("endwhile","endfunction","endif","endfor",
                       "endswitch","end_try_catch"): # octave
            t.type = "END_STMT"
            return t
        if t.value == "end":
            if t.lexer.parens > 0 or t.lexer.brackets > 0 or t.lexer.braces > 0:
                t.type = "END_EXPR"
            else:
                t.type = "END_STMT"
        else:
            t.type = reserved.get(t.value,"IDENT")
            if t.type != "IDENT" and t.lexer.lexdata[t.lexer.lexpos]=="'":
                t.lexer.begin("afterkeyword")
        return t


    def t_LPAREN(t):
        r"\("
        t.lexer.parens += 1
        return t

    def t_RPAREN(t):
        r"\)"
        t.lexer.parens -= 1
        return t

    @TOKEN(ws0+r"\]")
    def t_RBRACKET(t): # compare w t_LBRACKET
        t.lexer.lineno += t.value.count("\n")
        t.lexer.brackets -= 1
        if t.lexer.brackets+t.lexer.braces == 0:
            t.lexer.begin("INITIAL")
        return t

    @TOKEN(r"\["+ws0)
    def t_LBRACKET(t): # compare w t_SEMI
        t.lexer.lineno += t.value.count("\n")
        t.lexer.brackets += 1
        if t.lexer.brackets+t.lexer.braces == 1:
            t.lexer.begin("matrix")
        return t

    # maybe we need a dedicated CELLARRAY state ???
    @TOKEN(ws0+r"\}")
    def t_RBRACE(t):
        t.lexer.lineno += t.value.count("\n")
        t.lexer.braces -= 1
        if t.lexer.braces+t.lexer.brackets == 0:
            t.lexer.begin("INITIAL")
        return t

    @TOKEN(r"\{"+ws0)
    def t_LBRACE(t):
        t.lexer.lineno += t.value.count("\n")
        t.lexer.braces += 1
        if t.lexer.brackets+t.lexer.braces == 1:
            t.lexer.begin("matrix")
        return t

    @TOKEN(r","+ws0)
    def t_COMMA(t):  # eating spaces is important inside brackets
        t.lexer.lineno += t.value.count("\n")
        if (t.lexer.brackets == 0 and
            t.lexer.parens == 0 and
            t.lexer.braces == 0):
            t.type = "SEMI"
            return t
        return t

    @TOKEN(r"\;"+ws0)
    def t_SEMI(t):
        t.lexer.lineno += t.value.count("\n")
#        if t.lexer.brackets or t.lexer.braces > 0:
#            t.type = "CONCAT"
        return t

    def t_NUMBER(t):
        r"(0x[0-9A-Fa-f]+)|((\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?[ij]?)"
        if t.value[-1] == 'i':
            t.value = t.value[:-1]+'j'
        t.value = eval(t.value)
        return t

    def t_NEWLINE(t):
        r'\n+'
        t.lexer.lineno += len(t.value)
        if not t.lexer.parens and not t.lexer.braces:
            t.value = ";"
            t.type = "SEMI"
            return t

    def t_comment(t):
        r"(%|\#).*"
        pass


#    @TOKEN(ws+r"(?=[-+]\S)")    
#    def t_matrix_WHITESPACE(t):
#        #r"\s+(?=[-+]\S)"
#        # Whitespace, followed by + or - followed by anything but whitespace
#        t.lexer.lineno += t.value.count("\n")
#        t.type = "COMMA"
#        return t

    @TOKEN(r"(?<=\w)" + ws1 + r"(?=\()")
    def t_matrix_BAR(t):
        # Consume whitespace which follows end of name
        # and is followed a left paren.  This properly handles
        # a space between a func name and the arguments
        pass

    tend = r"(?<=[])}'\".]|\w)"
    tbeg = r"(?=[-+]?([[({'\"]|\w|\.\d))"
    @TOKEN(tend+ws1+tbeg)
    def t_matrix_FOO(t):
        # In matrix state, consume whitespace separating two
        # terms and return a fake COMMA token.  This allows
        # parsing [1 2 3] as if it was [1,2,3].  Handle
        # with care: [x + y] vs [x +y] 
        #
        # A term T is
        # (a) a name or a number
        # (b) literal string using single or doble quote
        # (c) (T) or [T] or {T} or T' or +T or -T
        #
        # Terms end with
        # (1) an alphanumeric charater \w
        # (2) single quote (in octave also double-quote)
        # (3) right parenthesis, bracket, or brace
        # (4) a dot (after a number, such as 3. 
        #
        # The pattern for whitespace accounts for ellipsis as a
        # whitespace, and for the trailing junk.
        #
        # Terms start with 
        # (1) an alphanumeric character
        # (2) a single or double quote,
        # (3) left paren, bracket, or brace and finally
        # (4) a dot before a digit, such as .3  .

        # TODO: what about curly brackets ???
        # TODO: what about dot followed by a letter, as in field
        #   [foo  .bar]
        
        t.lexer.lineno += t.value.count("\n")
        t.type = "COMMA"
        return t

    def t_ELLIPSIS(t):
        r"\.\.\..*\n"
        t.lexer.lineno += 1
        pass

    def t_SPACES(t):
        r"(\\\n|[ \t\r])+"
        pass

    def t_error(t):
        column=t.lexer.lexpos - t.lexer.lexdata.rfind("\n",0,t.lexer.lexpos)
        raise IllegalCharacterError(t.lineno,column,t.value[0])

    lexer = lex.lex(reflags=re.I)
    lexer.brackets = 0  # count open square brackets
    lexer.parens = 0    # count open parentheses
    lexer.braces = 0    # count open curly braces
    return lexer

if __name__ == "__main__":
    lexer = new()
    while 1:
        line = raw_input("=>> ")
        if not line:
            break
        while line[-1] == "\\":
            line = line[:-1] + raw_input("... ")
        print len(line), [c for c  in line]
        lexer.input(line)
        for tok in lexer:
            print len(str(tok.value)), tok



import numpy as np
import sys

class sparsearray(dict):
    def __init__(self,input_array=[[]]):
        #import pdb;pdb.set_trace()
        dict.__init__(self)
        a = np.atleast_2d(input_array)
        self.dtype = a.dtype
        self.ndim = a.ndim
        self._shape = None
        if not a.size:
            return
        it = np.nditer(a, flags=['multi_index'])
        while not it.finished:
            index = tuple([i+1 for i in it.multi_index])
            self.setdefault(index,it[0].item())
            it.iternext()

    @property
    def shape(self):
        #import pdb;pdb.set_trace()
        if self._shape is None:
            s = [0] * self.ndim
            for key in self.keys():
                for i,k in enumerate(key):
                    s[i] = max(k,s[i])
            self._shape = tuple(s)
        return self._shape

    def todense(self):
        a = np.zeros(self.shape,dtype=self.dtype)
        for key,value in self.iteritems():
            key = tuple([i-1 for i in key])
            a.__setitem__(key,value)
        return a

    def __str__(self):
        return str(self.todense())

    def __repr__(self):
        return repr(self.todense())

    def copy(self):
        #return copy.copy(self)
        return self.todense()

    def __setitem__(self,index,value):
        if np.isscalar(value):
            for key in self.iterkeys(index):
                dict.__setitem__(self,key,value)
            self._shape = None
        else:
            raise NotImplementedError

    def __getslice__(self,i,j):
        if j == sys.maxint:
            j = None
        return self.__getitem__(slice(i,j,None))

    def __getitem__(self,index):
        try:
            #a = [dict.__getitem__(self,key) for key in self.iterkeys(index)]
            a = [self.get(key,0) for key in self.iterkeys(index)]
            if len(a) == 1:
                return a[0]
        except ValueError:
            raise IndexError # out of bound rhs indexing
        #return a
        #return sparsearray([a])
        return np.array(a)

    def iterkeys(self,index):
        #import pdb; pdb.set_trace()
        if not isinstance(index,tuple) and self.shape[0] == 1:
            index = (1,index)
        if isinstance(index, int):
            key = np.unravel_index(index-1, self.shape, order='F')
            yield tuple(k+1 for k in key)
        elif isinstance(index,slice):
            index = range((index.start or 1)-1,
                          index.stop or np.prod(self.shape),
                          index.step or 1)
            for key in np.transpose(np.unravel_index(index, self.shape, order='F')): # 0-based
                yield tuple(k+1 for k in key)
        elif isinstance(index,(list,np.ndarray)):
            index = np.asarray(index)-1
            for key in np.transpose(np.unravel_index(index, self.shape, order='F')):
                yield tuple(k+1 for k in key)
        else:
            assert isinstance(index,tuple),index.__class__
            indices = []  # 1-based
            for i,ix in enumerate(index):
                if isinstance(ix,slice):
                    indices.append(np.arange((ix.start or 1),
                                             (ix.stop  or self.shape[i]) + 1,
                                             ix.step   or 1,
                                             dtype=int))
                else:
                    indices.append(np.asarray(ix))
            assert len(index) == 2
            indices[0].shape = (-1,1)
            for key in np.broadcast(*indices):
                yield tuple(map(int,key))




import unittest
import lexer

class TestLexer(unittest.TestCase):
    def setUp(self):
        self.lexer = lexer.new()
        
    def test010(self):
        """Reserved words are not reserved as field names"""
        self.lexer.input("for foo.for")
        t = [tok.type for tok in self.lexer]
        u = ["FOR","IDENT","FIELD"]
        self.assertEqual(t,u)

    def test020(self):
        """Fields are recognized beyond continuation lines"""
        self.lexer.input("foo ... hello \n\t .\t  ... world\n \t ... \n bar")
        t = [tok.type for tok in self.lexer]
        u = ["IDENT","FIELD"]
        self.assertEqual(t,u)

    ### commands are now recognized in the parser
    # def test_t03(self):
    #     """Name following another name is a command.  Names might be
    #     separated by whitespace, but must be on the same line . Once
    #     we detect a command, every name (including keywords) is
    #     reported as a string.
    #     """
    #     self.lexer.input("foo bar for")
    #     t = [tok.type for tok in self.lexer]
    #     u = ["COMMAND","STRING","STRING"]
    #     self.assertEqual(t,u)        

    # def test_t03a(self):
    #     self.lexer.input("dir '/windows'")
    #     t = [tok.type for tok in self.lexer]
    #     u = ["COMMAND","STRING"]
    #     self.assertEqual(t,u)        

    def test030(self):
        self.lexer.input("for bar =")
        t = [tok.type for tok in self.lexer]
        u = ["FOR", "IDENT", "EQ"]
        self.assertEqual(t,u)
        
    def test040(self):
        "Ignore anything from ... to end of line"
        self.lexer.input("foo ... hello world \n bar")
        # line continuation
        t = [tok.type for tok in self.lexer]
        u = ["IDENT", "IDENT"]
        self.assertEqual(t,u)

    def test042(self):
        "backslash-newline continues a line"
        s = "foo \\\n bar ... hello\n bzz"
        #print 43,`s`
        #print '#', s.count('\n')
        self.lexer.input(s)
        # line continuation
        t = [tok.type for tok in self.lexer]
        u = ["IDENT", "IDENT", "IDENT"]
        self.assertEqual(t,u)

    def test044(self):
        "backslash-newline continues a line"
        self.lexer.input(r'puts ("hel\"zvvzcvnzvzlozz")')
        # line continuation
        t = [tok.type for tok in self.lexer]
        u = ["IDENT", "LPAREN", "STRING","RPAREN"]
        self.assertEqual(t,u)

    def test050(self):
        self.lexer.input("clear all, figure on")
        t = [tok.type for tok in self.lexer]
        u = ["IDENT","IDENT","SEMI","IDENT","IDENT"]
        self.assertEqual(t,u)

    def test060(self):
        self.lexer.input("dir '.', for i=foo")
        t = [tok.type for tok in self.lexer]
        u = ["IDENT","STRING","SEMI","FOR","IDENT","EQ","IDENT"]
        self.assertEqual(t,u)

    def test070(self):
        """Line continuation is allowed between command arguments,
        but not between the command and the first argument."""
        self.lexer.input("dir 'hello' ... adfadfsda \n fooobar")
        t = [tok.type for tok in self.lexer]
        u = ["IDENT","STRING","IDENT"]
        self.assertEqual(t,u)  
        
    def test080(self):
        """Terms may start and end with a dot, when a floating
        point number starts or ends with a decimal point"""
        self.lexer.input("[1. .2]")
        t = [tok.type for tok in self.lexer]
        u = ["LBRACKET","NUMBER","COMMA","NUMBER","RBRACKET"]
        self.assertEqual(t,u)  

    def test090(self):
        """Ops that start with a dot (./ etc) should not
        be understood as terms"""
        self.lexer.input("[1. ./ .2]")
        t = [tok.type for tok in self.lexer]
        u = ["LBRACKET","NUMBER","DOTDIV","NUMBER","RBRACKET"]
        self.assertEqual(t,u)  

    def test100(self):
        "Complex constants, such as 1i and 1j"
        self.lexer.input("1i+1j")
        t = [tok.type for tok in self.lexer]
        u = ["NUMBER","PLUS","NUMBER"]
        self.assertEqual(t,u)  
        
    def test110(self):
        "Quotes and backslashes in matlab strings"
        self.lexer.input(r"'hello''world'")
        tok = self.lexer.next()
        self.assertEqual(tok.value,r"hello'world")
    
    def test112(self):
        "Quotes and backslashes in octave strings"
        self.lexer.input(r'"hello\"world"')
        tok = self.lexer.next()
        self.assertEqual(tok.value,r'hello"world')

    def test114(self):
        "Quotes and backslashes in octave strings"
        self.lexer.input('''"hello\
world"''')
        tok = self.lexer.next()
        self.assertEqual(tok.value,r'helloworld')

    def test116(self):
        "Quotes and backslashes in octave strings"
        self.lexer.input(r'"hello\n"')
        tok = self.lexer.next()
        self.assertEqual(tok.value,'hello\n')

    def test118(self):
        "Quotes and backslashes in octave strings"
        self.lexer.input(r'"hello\\world"')
        tok = self.lexer.next()
        self.assertEqual(tok.value,r'hello\world')

    def test120(self):
        "d5ef: cell arrays nested in regular arrays"
        self.lexer.input(r"[foo{i} bar]")
        t = [tok.type for tok in self.lexer]
        u = ["LBRACKET","IDENT","LBRACE","IDENT","RBRACE","COMMA","IDENT","RBRACKET"]
        self.assertEqual(t,u)  

    def test130(self):
        "d5ef: Another one"
        self.lexer.input("[y / (x - x) ]")
        t = [tok.type for tok in self.lexer]
        u = ["LBRACKET","IDENT","DIV","LPAREN","IDENT","MINUS","IDENT","RPAREN","RBRACKET"]
        self.assertEqual(t,u)  

    def test140(self):
        "quote immediatly following a keyword is always a string"
        self.lexer.input("case'abc'")
        t = [tok.type for tok in self.lexer]
        u = ["CASE","STRING"]
        self.assertEqual(t,u)  

    def test150(self):
        s = "g = (x < 3/8) .* (7/8 * x)\\\n + (x >= 3/8 & x < 3/4) .* (29/24 * x - 1/8)\\\n + (x >= 3/4) .* (7/8 * x + 1/8)"
        self.lexer.input(s)
        toklist = [tok.type for tok in self.lexer]
        self.assertEqual(True, "BACKSLASH" not in toklist)
        self.assertEqual(True, "SEMI" not in toklist)

    def test160(self):
        "comment must leave a newline"
        s = """if (ld > 127)   # convert 16 to 8 bit
               if (ld < 16384)
            """
        self.lexer.input(s)
        t = [tok.type for tok in self.lexer]
        self.assertEqual("SEMI",t[6])

if __name__ == "__main__":
    unittest.main()



from setuptools import setup

setup(
    author = 'Victor Leikehman',
    author_email = 'victorlei@gmail.com',
    description = 'Matlab to Python converter',
    license = 'MIT',
    keywords = 'convert translate matlab octave python',
    url = 'https://github.com/victorlei/smop',
    download_url = 'https://github.com/victorlei/smop/archive/master.zip',
    name = 'smop',
    version = '0.25.4',
    entry_points = { 'console_scripts': [ 'smop = smop.main:main', ], },
    packages = ['smop'],
    #package_dir = {'':'src'},
    #test_suite = "smop.testsuite.test_lexer",
    #include_package_data = True,
    install_requires = ['numpy', 'scipy', 'networkx'],
)



# SMOP compiler -- Simple Matlab/Octave to Python compiler
# Copyright 2011-2013 Victor Leikehman

import sys,cPickle,glob,os
import getopt,re
import lexer,parse,resolve,backend,options,node,graphviz
import networkx as nx
import readline

__version__ = "0.25.4"

def usage():
    print "SMOP compiler version " + __version__
    print """Usage: smop [options] file-list
    Options:
    -V --version
    -X --exclude=FILES      Ignore files listed in comma-separated list FILES
    -d --dot=REGEX          For functions whose names match REGEX, save debugging
                            information in "dot" format (see www.graphviz.org).
                            You need an installation of graphviz to use --dot
                            option.  Use "dot" utility to create a pdf file.
                            For example: 
                                $ python main.py fastsolver.m -d "solver|cbest"
                                $ dot -Tpdf -o resolve_solver.pdf resolve_solver.dot
    -h --help
    -o --output=FILENAME    By default create file named a.py
    -o- --output=-          Use standard output
    -s --strict             Stop on the first error
    -v --verbose
"""

def main():
    try:
        opts, args = getopt.gnu_getopt(sys.argv[1:],
                                       "d:ho:vVsX:", 
                                       [
                                        "dot=",
                                        "exclude",
                                        "help",
                                        "output=",
                                        "strict",
                                        "verbose",
                                        "version",
                                       ])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)

    exclude_list = []
    output = None
    verbose = 0
    strict = 0
    dot = None

    for o, a in opts:
        if o in ("-s", "--strict"):
            strict = 1
        elif o in ("-d", "--dot"):
            dot = re.compile(a)
        elif o in ("-X", "--exclude"):
            exclude_list += a.split(",")
        elif o in ("-v", "--verbose"):
            verbose += 1
        elif o in ("-V", "--version"):
            print "SMOP compiler version " + __version__
            sys.exit()
        elif o in ("-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-o", "--output"):
            output = a
        else:
            assert False, "unhandled option"

    """
    if not args:
        usage()
        sys.exit()
    """
    if not args:
        symtab = {}
        print "? for help"
        while 1:
            try:
                buf = raw_input("=>> ")
                if not buf:
                    continue
                while buf[-1] == "\\":
                    buf = buf[:-1] + "\n" + raw_input("... ")
                #print buf
                if buf[0] == '?':
                    print '!a="def f(): \\n\\treturn 123"'
                    print "!exec a"
                    print "!print f"
                    print "!print f()"
                    print "!reload(backend)"
                    print "=>> function t=foo(a) \\"
                    print "... t=123"
                    print "!exec foo(3)"

                    continue
                if buf[0] == '!':
                    try:
                        exec buf[1:]
                    except:
                        print "eh?"
                    continue
                t = parse.parse(buf if buf[-1]=='\n' else buf+'\n')
                if not t:
                    continue
            except EOFError:
                return
            resolve.resolve(t,symtab)
            _ = backend.backend(t)
            print _

    if not output:
        output = "a.py"
    fp = open(output,"w") if output != "-" else sys.stdout
    print >> fp, "# Autogenerated with SMOP version " + __version__
    print >> fp, "# " + " ".join(sys.argv)
    print >> fp, "from __future__ import division"
    print >> fp, "from runtime import *" 

    for pattern in args:
        for filename in glob.glob(os.path.expanduser(pattern)):
            if not filename.endswith(".m"):
                print "\tIngored file: '%s'" % filename
                continue
            if os.path.basename(filename) in exclude_list:
                print "\tExcluded file: '%s'" % filename
                continue
            if verbose:
                print filename
            buf = open(filename).read().replace("\r\n","\n")
            func_list = parse.parse(buf if buf[-1]=='\n' else buf+'\n',filename)
            if not func_list and strict:
                sys.exit(-1)

            for func_obj in func_list: 
                try:
                    func_name = func_obj.head.ident.name
                    if verbose:
                        print "\t",func_name
                except AttributeError:
                    if verbose:
                        print "\tJunk ignored"
                    if strict:
                        sys.exit(-1)
                    continue
                fp0 = open("parse_"+func_name+".dot","w") if dot and dot.match(func_name) else None
                if fp0:
                    graphviz.graphviz(func_obj,fp0)
                if options.do_resolve:
                    G = resolve.resolve(func_obj)

            for func_obj in func_list:
                s = backend.backend(func_obj)
                print >> fp, s

if __name__ == "__main__":
    main()
